@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -61,7 +61,6 @@ static int apicinstance_test1(fwts_framework *fw)
 		fwts_failed(fw, LOG_LEVEL_HIGH, "MultipleAPICMADT",
 			"Found %d APIC/MADT tables, the kernel expects just one.",
 			count);
-		fwts_tag_failed(fw, FWTS_TAG_ACPI_INVALID_TABLE);
 		snprintf(buffer, sizeof(buffer), " (or up to %d)", count);
 		fwts_log_advice(fw,
 			"If you find any APIC issues, perhaps try using "
@@ -74,12 +73,12 @@ static int apicinstance_test1(fwts_framework *fw)
 }
 
 static fwts_framework_minor_test apicinstance_tests[] = {
-	{ apicinstance_test1, "Check single instance of APIC/MADT table." },
+	{ apicinstance_test1, "Test for single instance of APIC/MADT table." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops apicinstance_ops = {
-	.description = "Check for single instance of APIC/MADT table.",
+	.description = "Test for single instance of APIC/MADT table.",
 	.minor_tests = apicinstance_tests
 };
 
