@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006, Intel Corporation
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This file was originally part of the Linux-ready Firmware Developer Kit
  *
@@ -264,7 +264,6 @@ static int dmar_acpi_table_check(fwts_framework *fw)
 	if (table->length <= DMAR_HEADER_SIZE) {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "InvalidDMAR",
 			"Invalid DMAR ACPI table.");
-		fwts_tag_failed(fw, FWTS_TAG_ACPI_INVALID_TABLE);
 		return FWTS_ERROR;
 	}
 
@@ -328,12 +327,12 @@ static int dmar_test1(fwts_framework *fw)
 }
 
 static fwts_framework_minor_test dmar_tests[] = {
-	{ dmar_test1, "Check DMA Remapping." },
+	{ dmar_test1, "DMA Remapping test." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops dmar_ops = {
-	.description = "Check sane DMA Remapping (VT-d).",
+	.description = "DMA Remapping (VT-d) test.",
 	.minor_tests = dmar_tests
 };
 
