@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -56,14 +56,13 @@ static int power_button_test1(fwts_framework *fw)
 		}
 		fwts_printf(fw, "Waiting %2.2d/20\r", 20-i);
 	}
-	if (matching == 0) {
+	if (matching == 0)
 		fwts_failed(fw, LOG_LEVEL_HIGH, "NoPowerButtonEvents",
 			"Did not detect any ACPI power buttons events while waiting for power button to be pressed.");
-		fwts_tag_failed(fw, FWTS_TAG_ACPI_EVENT);
-	} else  {
+	else  {
 		char button[4096];
 		memset(button, 0, sizeof(button));
-		sscanf(buffer, "%*s %s", button);
+		sscanf(buffer, "%*s %4095s", button);
 
 		fwts_passed(fw, "Detected %s power button event.", button);
 	}
