@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,10 @@
 #include <unistd.h>
 #include <time.h>
 
-#define PM_SUSPEND "pm-suspend"
+#define PM_SUSPEND 		"pm-suspend"
+#define PM_SUSPEND_HYBRID 	"pm-suspend-hybrid"
+#define FWTS_SUSPEND		"FWTS_SUSPEND"
+#define FWTS_RESUME		"FWTS_RESUME"
 
 static int  s3_multiple = 1;		/* number of s3 multiple tests to run */
 static int  s3_min_delay = 0;		/* min time between resume and next suspend */
@@ -40,17 +43,14 @@ static bool s3_device_check = false;	/* check for device config changes */
 static char *s3_quirks = NULL;		/* Quirks to be passed to pm-suspend */
 static int  s3_device_check_delay = 15;	/* Time to sleep after waking up and then running device check */
 static bool s3_min_max_delay = false;
+static float s3_suspend_time = 15.0;	/* Maximum allowed suspend time */
+static float s3_resume_time = 15.0;	/* Maximum allowed resume time */
+static bool s3_hybrid = false;
 
 static int s3_init(fwts_framework *fw)
 {
-	int ret;
-
 	/* Pre-init - make sure wakealarm works so that we can wake up after suspend */
-	if (fwts_klog_clear()) {
-		fwts_log_error(fw, "Cannot clear kernel log.");
-		return FWTS_ERROR;
-	}
-	if ((ret = fwts_wakealarm_test_firing(fw, 1))) {
+	if (fwts_wakealarm_test_firing(fw, 1) != FWTS_OK) {
 		fwts_log_error(fw, "Cannot automatically wake machine up - aborting S3 test.");
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "BadWakeAlarmS3",
 			"Check if wakealarm works reliably for S3 tests.");
@@ -66,7 +66,6 @@ static int s3_do_suspend_resume(fwts_framework *fw,
 	int delay,
 	int percent)
 {
-	fwts_list *output;
 	fwts_hwinfo hwinfo1, hwinfo2;
 	int status;
 	int duration;
@@ -77,16 +76,17 @@ static int s3_do_suspend_resume(fwts_framework *fw,
 	char *quirks;
 	char buffer[80];
 
-	fwts_klog_clear();
-
 	if (s3_device_check)
 		fwts_hwinfo_get(fw, &hwinfo1);
 
-
-
 	/* Format up pm-suspend command with optional quirking arguments */
-	if ((command = fwts_realloc_strcat(NULL, PM_SUSPEND)) == NULL)
-		return FWTS_OUT_OF_MEMORY;
+	if (s3_hybrid) {
+		if ((command = fwts_realloc_strcat(NULL, PM_SUSPEND_HYBRID)) == NULL)
+			return FWTS_OUT_OF_MEMORY;
+	} else {
+		if ((command = fwts_realloc_strcat(NULL, PM_SUSPEND)) == NULL)
+			return FWTS_OUT_OF_MEMORY;
+	}
 
 	if (s3_quirks) {
 		if ((command = fwts_realloc_strcat(command, " ")) == NULL)
@@ -107,16 +107,18 @@ static int s3_do_suspend_resume(fwts_framework *fw,
 	/* Do S3 here */
 	fwts_progress_message(fw, percent, "(Suspending)");
 	time(&t_start);
-	(void)fwts_pipe_exec(command, &output, &status);
+	(void)fwts_klog_write(fw, "Starting fwts suspend\n");
+	(void)fwts_klog_write(fw, FWTS_SUSPEND "\n");
+	(void)fwts_exec(command, &status);
+	(void)fwts_klog_write(fw, FWTS_RESUME "\n");
+	(void)fwts_klog_write(fw, "Finished fwts resume\n");
 	time(&t_end);
 	fwts_progress_message(fw, percent, "(Resumed)");
-	fwts_text_list_free(output);
 	free(command);
 
 	duration = (int)(t_end - t_start);
 	fwts_log_info(fw, "pm-suspend returned %d after %d seconds.", status, duration);
 
-
 	if (s3_device_check) {
 		int i;
 
@@ -142,7 +144,6 @@ static int s3_do_suspend_resume(fwts_framework *fw,
 		(*pm_errors)++;
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "ShortSuspend",
 			"Unexpected: S3 slept for %d seconds, less than the expected %d seconds.", duration, delay);
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 	}
 	fwts_progress_message(fw, percent, "(Checking for errors)");
 	if (duration > (delay*2)) {
@@ -169,38 +170,115 @@ static int s3_do_suspend_resume(fwts_framework *fw,
 		fwts_failed(fw, LOG_LEVEL_HIGH, "PMActionFailedPreS3",
 			"pm-action failed before trying to put the system "
 			"in the requested power saving state.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 	} else if (status == 128) {
 		(*pm_errors)++;
 		fwts_failed(fw, LOG_LEVEL_HIGH, "PMActionPowerStateS3",
 			"pm-action tried to put the machine in the requested "
 			"power state but failed.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 	} else if (status > 128) {
 		(*pm_errors)++;
 		fwts_failed(fw, LOG_LEVEL_HIGH, "PMActionFailedS3",
 			"pm-action encountered an error and also failed to "
 			"enter the requested power saving state.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 	}
 
 	return FWTS_OK;
 }
 
-static int s3_check_log(fwts_framework *fw, int *errors, int *oopses, int *warn_ons)
+static int s3_scan_times(
+	fwts_framework *fw,
+	fwts_list *klog,
+	int *suspend_too_long,
+	int *resume_too_long)
+{
+	fwts_list_link *item;
+
+	double s3_suspend_start = -1.0, s3_suspend_finish = -1.0;
+	double s3_resume_start = -1.0, s3_resume_finish = -1.0;
+	double previous_ts = -1.0, ts = 0.0;
+
+	fwts_list_foreach(item, klog) {
+		char *txt = (char *)item->data;
+		char *bracket = strchr(txt, '[');
+		size_t len = strlen(txt);
+
+		if (len < 15 || bracket == NULL)
+			continue;
+
+		previous_ts = ts;
+
+		/* Get log time stamp */
+		sscanf(bracket + 1, "%lf", &ts);
+		if (strstr(txt, FWTS_SUSPEND)) {
+			s3_suspend_start = ts;
+			continue;
+		}
+		if (strstr(txt, FWTS_RESUME)) {
+			s3_resume_finish = ts;
+			continue;
+		}
+		/* This may be the last message we see from the kernel */
+		if (strstr(txt, "PM: Saving platform NVS memory")) {
+			s3_suspend_finish = ts;
+			continue;
+		}
+		/* And this may appear even later */
+		if (strstr(txt, "smpboot: CPU") && strstr(txt, "is now offline")) {
+			s3_suspend_finish = ts;
+			continue;
+		}
+
+		if (strstr(txt, "ACPI: Low-level resume complete")) {
+			s3_resume_start = ts;
+			if (s3_suspend_finish < 0.0)
+				s3_suspend_finish = previous_ts;
+			continue;
+		}
+		if (s3_resume_start < 0.0 && strstr(txt, "ACPI: Waking up from system sleep state S3")) {
+			s3_resume_start = ts;
+			if (s3_suspend_finish < 0.0)
+				s3_suspend_finish = previous_ts;
+			continue;
+		}
+		if (strstr(txt, "Restarting tasks ... done")) {
+			s3_resume_finish = ts;
+			break;
+		}
+	}
+
+	fwts_log_info(fw, "Suspend/Resume Timings:");
+	if (s3_suspend_start > 0.0 && s3_suspend_finish > 0.0) {
+		fwts_log_info_verbatum(fw, "  Suspend: %.3f seconds.",
+			s3_suspend_finish - s3_suspend_start);
+		if (s3_suspend_finish - s3_suspend_start > s3_suspend_time)
+			(*suspend_too_long)++;
+	} else
+		fwts_log_info_verbatum(fw, "  Could not determine time to suspend.");
+
+	if (s3_resume_start > 0.0 && s3_resume_finish > 0.0) {
+		fwts_log_info_verbatum(fw, "  Resume:  %.3f seconds.",
+			s3_resume_finish - s3_resume_start);
+		if (s3_resume_finish - s3_resume_start > s3_resume_time)
+			(*resume_too_long)++;
+	} else
+		fwts_log_info_verbatum(fw, "  Could not determine time to resume.");
+
+	return FWTS_OK;
+}
+
+static int s3_check_log(
+	fwts_framework *fw,
+	fwts_list *klog,
+	int *errors,
+	int *oopses,
+	int *warn_ons,
+	int *suspend_too_long,
+	int *resume_too_long)
 {
-	fwts_list *klog;
 	int error;
 	int oops;
 	int warn_on;
 
-	if ((klog = fwts_klog_read()) == NULL) {
-		fwts_log_error(fw, "Cannot read kernel log.");
-		fwts_failed(fw, LOG_LEVEL_MEDIUM, "KlogCheckS3",
-			"Unable to check kernel log for S3 suspend/resume test.");
-		return FWTS_ERROR;
-	}
-
 	if (fwts_klog_pm_check(fw, NULL, klog, &error))
 		fwts_log_error(fw, "Error parsing kernel log.");
 	*errors += error;
@@ -215,7 +293,7 @@ static int s3_check_log(fwts_framework *fw, int *errors, int *oopses, int *warn_
 	*oopses += oops;
 	*warn_ons += warn_on;
 
-	fwts_klog_free(klog);
+	s3_scan_times(fw, klog, suspend_too_long, resume_too_long);
 
 	return FWTS_OK;
 }
@@ -228,6 +306,8 @@ static int s3_test_multiple(fwts_framework *fw)
 	int pm_errors = 0;
 	int klog_oopses = 0;
 	int klog_warn_ons = 0;
+	int suspend_too_long = 0;
+	int resume_too_long = 0;
 	int awake_delay = s3_min_delay * 1000;
 	int delta = (int)(s3_delay_delta * 1000.0);
 
@@ -237,15 +317,29 @@ static int s3_test_multiple(fwts_framework *fw)
 	for (i=0; i<s3_multiple; i++) {
 		struct timeval tv;
 		int percent = (i * 100) / s3_multiple;
-
+		fwts_list *klog_pre, *klog_post, *klog_diff;
 		fwts_log_info(fw, "S3 cycle %d of %d\n",i+1,s3_multiple);
 
+		if ((klog_pre = fwts_klog_read()) == NULL)
+			fwts_log_error(fw, "Cannot read kernel log.");
+
 		if (s3_do_suspend_resume(fw, &hw_errors, &pm_errors, s3_sleep_delay, percent) == FWTS_OUT_OF_MEMORY) {
 			fwts_log_error(fw, "S3 cycle %d failed - out of memory error.", i+1);
+			fwts_klog_free(klog_pre);
 			break;
 		}
+
+		if ((klog_post = fwts_klog_read()) == NULL)
+			fwts_log_error(fw, "Cannot re-read kernel log.");
+
 		fwts_progress_message(fw, percent, "(Checking logs for errors)");
-		s3_check_log(fw, &klog_errors, &klog_oopses, &klog_warn_ons);
+		klog_diff = fwts_klog_find_changes(klog_pre, klog_post);
+		s3_check_log(fw, klog_diff, &klog_errors, &klog_oopses, &klog_warn_ons,
+			&suspend_too_long, &resume_too_long);
+
+		fwts_klog_free(klog_pre);
+		fwts_klog_free(klog_post);
+		fwts_list_free(klog_diff, NULL);
 
 		if (!s3_device_check) {
 			char buffer[80];
@@ -294,13 +388,30 @@ static int s3_test_multiple(fwts_framework *fw)
 	else
 		fwts_passed(fw, "No kernel WARN_ON warnings detected.");
 
-
 	if ((klog_errors + pm_errors + hw_errors + klog_oopses) > 0) {
 		fwts_log_info(fw, "Found %d errors and %d oopses doing %d suspend/resume cycle(s).",
 			klog_errors + pm_errors + hw_errors, klog_oopses, s3_multiple);
 	} else
 		fwts_passed(fw, "Found no errors doing %d suspend/resume cycle(s).", s3_multiple);
 
+	if (suspend_too_long > 0)
+		fwts_failed(fw, LOG_LEVEL_MEDIUM, "SuspendTooLong",
+			"%d suspend%s took longer than %.2f seconds.",
+			suspend_too_long, resume_too_long == 1 ? "" : "s",
+			s3_suspend_time);
+	else
+		fwts_passed(fw, "All suspends took less than %.2f seconds.",
+			s3_suspend_time);
+
+	if (resume_too_long > 0)
+		fwts_failed(fw, LOG_LEVEL_MEDIUM, "ResumeTooLong",
+			"%d resume%s took longer than %.2f seconds.",
+			resume_too_long, resume_too_long == 1 ? "" : "s",
+			s3_resume_time);
+	else
+		fwts_passed(fw, "All resumes took less than %.2f seconds.",
+			s3_resume_time);
+
 	return FWTS_OK;
 }
 
@@ -336,6 +447,14 @@ static int s3_options_check(fwts_framework *fw)
 		fprintf(stderr, "Cannot use --s3-min-delay, --s3-max-delay, --s3-delay-delta as well as --s3-device-check, --s3-device-check-delay.\n");
 		return FWTS_ERROR;
 	}
+	if (s3_suspend_time < 0.0) {
+		fprintf(stderr, "--s3-suspend-time too small.\n");
+		return FWTS_ERROR;
+	}
+	if (s3_resume_time < 0.0) {
+		fprintf(stderr, "--s3-resume-time too small.\n");
+		return FWTS_ERROR;
+	}
 	return FWTS_OK;
 }
 
@@ -376,6 +495,15 @@ static int s3_options_handler(fwts_framework *fw, int argc, char * const argv[],
 			s3_device_check_delay = atoi(optarg);
 			s3_device_check = true;
 			break;
+		case 8:
+			s3_suspend_time = atof(optarg);
+			break;
+		case 9:
+			s3_resume_time = atof(optarg);
+			break;
+		case 10:
+			s3_hybrid = true;
+			break;
 		}
 	}
 	return FWTS_OK;
@@ -390,6 +518,9 @@ static fwts_option s3_options[] = {
 	{ "s3-device-check",	"", 0, "Check differences between device configurations over a S3 cycle. Note we add a default of 15 seconds to allow wifi to re-associate.  Cannot be used with --s3-min-delay, --s3-max-delay and --s3-delay-delta." },
 	{ "s3-quirks",		"", 1, "Comma separated list of quirk arguments to pass to pm-suspend." },
 	{ "s3-device-check-delay", "", 1, "Sleep N seconds before we run a device check after waking up from suspend. Default is 15 seconds, e.g. --s3-device-check-delay=20" },
+	{ "s3-suspend-time",	"", 1, "Maximum expected suspend time in seconds, e.g. --s3-suspend-time=3.5" },
+	{ "s3-resume-time", 	"", 1, "Maximum expected resume time in seconds, e.g. --s3-resume-time=5.1" },
+	{ "s3-hybrid",		"", 0, "Run S3 with hybrid sleep, i.e. saving system states as S4 does." },
 	{ NULL, NULL, 0, NULL }
 };
 
