@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2013 Canonical
+ * Copyright (C) 2011-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -172,8 +172,6 @@ static int s3power_test(fwts_framework *fw)
 	uint32_t capacity_before_mWh;
 	uint32_t capacity_after_mWh;
 
-	fwts_list *output;
-
 	if (s3power_wait_for_adapter_offline(fw, &offline) == FWTS_ERROR) {
 		fwts_log_error(fw, "Cannot check if machine is running on battery, aborting test.");
 		return FWTS_ABORTED;
@@ -190,10 +188,9 @@ static int s3power_test(fwts_framework *fw)
 	/* Do S3 here */
 	fwts_progress_message(fw, 100, "(Suspending)");
 	time(&t_start);
-	(void)fwts_pipe_exec(PM_SUSPEND, &output, &status);
+	(void)fwts_exec(PM_SUSPEND, &status);
 	time(&t_end);
 	fwts_progress_message(fw, 100, "(Resumed)");
-	fwts_text_list_free(output);
 
 	s3power_get_remaining_capacity(fw, &capacity_after_mAh, &capacity_after_mWh);
 
@@ -203,12 +200,10 @@ static int s3power_test(fwts_framework *fw)
 	duration = (int)(t_end - t_start);
 	fwts_log_info(fw, "pm-suspend returned %d after %d seconds.", status, duration);
 
-	if (duration < s3power_sleep_delay) {
+	if (duration < s3power_sleep_delay)
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "ShortSuspend",
 			"Unexpected: S3 slept for %d seconds, less than the expected %d seconds.",
 			duration, s3power_sleep_delay);
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
-	}
 	if (duration > (s3power_sleep_delay*2))
 		fwts_failed(fw, LOG_LEVEL_HIGH, "LongSuspend",
 			"Unexpected: S3 much longer than expected (%d seconds).", duration);
@@ -218,17 +213,14 @@ static int s3power_test(fwts_framework *fw)
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "PMActionFailedPreS3",
 			"pm-action failed before trying to put the system "
 			"in the requested power saving state.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 	} else if (status == 128) {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "PMActionPowerStateS3",
 			"pm-action tried to put the machine in the requested "
 			"power state but failed.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 	} else if (status > 128) {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "PMActionFailedS3",
 			"pm-action encountered an error and also failed to "
 			"enter the requested power saving state.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 	}
 
 	return FWTS_OK;
