@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,9 @@
 #include <unistd.h>
 
 #define PM_HIBERNATE	"pm-hibernate"
+#define FWTS_HIBERNATE	"FWTS_HIBERNATE"
+#define FWTS_RESUME	"FWTS_RESUME"
+
 
 #define FWTS_TRACING_BUFFER_SIZE	"/sys/kernel/debug/tracing/buffer_size_kb"
 
@@ -44,11 +47,6 @@ static int s4_init(fwts_framework *fw)
 {
 	fwts_list* swap_devs;
 
-	if (fwts_klog_clear()) {
-		fwts_log_error(fw, "Cannot clear kernel log.");
-		return FWTS_ERROR;
-	}
-
 	swap_devs = fwts_file_open_and_read("/proc/swaps");
 	if (fwts_text_list_strstr(swap_devs, "/dev/") == NULL) {
 		fwts_list_free(swap_devs, free);
@@ -99,8 +97,7 @@ static int s4_hibernate(fwts_framework *fw,
 	int *failed_alloc_image,
 	int percent)
 {
-	fwts_list *output;
-	fwts_list *klog;
+	fwts_list *klog_pre, *klog_post, *klog_diff;
 	fwts_hwinfo hwinfo1, hwinfo2;
 	int status;
 	int differences;
@@ -108,11 +105,6 @@ static int s4_hibernate(fwts_framework *fw,
 	char *quirks;
 	char buffer[80];
 
-	if (fwts_klog_clear()) {
-		fwts_log_error(fw, "S4 hibernate: Cannot clear kernel log.");
-		return FWTS_ERROR;
-	}
-
 	if (s4_device_check)
 		fwts_hwinfo_get(fw, &hwinfo1);
 
@@ -137,12 +129,21 @@ static int s4_hibernate(fwts_framework *fw,
 	fwts_wakealarm_trigger(fw, s4_sleep_delay);
 
 	/* Do s4 here */
+	if ((klog_pre = fwts_klog_read()) == NULL)
+		fwts_log_error(fw, "S4: hibernate: Cannot read kernel log.");
+
 	fwts_progress_message(fw, percent, "(Hibernating)");
-	(void)fwts_pipe_exec(command, &output, &status);
+	(void)fwts_klog_write(fw, "Starting fwts hibernate\n");
+	(void)fwts_klog_write(fw, FWTS_HIBERNATE "\n");
+	(void)fwts_exec(command, &status);
+	(void)fwts_klog_write(fw, FWTS_RESUME "\n");
+	(void)fwts_klog_write(fw, "Finished fwts resume\n");
 	fwts_progress_message(fw, percent, "(Resumed)");
-	fwts_text_list_free(output);
 	free(command);
 
+	if ((klog_post = fwts_klog_read()) == NULL)
+		fwts_log_error(fw, "S4: hibernate: Cannot re-read kernel log.");
+
 	if (s4_device_check) {
 		int i;
 
@@ -166,12 +167,8 @@ static int s4_hibernate(fwts_framework *fw,
 
 	fwts_progress_message(fw, percent, "(Checking for errors)");
 
-	if ((klog = fwts_klog_read()) == NULL) {
-		fwts_log_error(fw, "S4: hibernate: Cannot read kernel log.");
-		return FWTS_ERROR;
-	}
-
-	s4_check_log(fw, klog, klog_errors, klog_oopses, klog_warn_ons);
+	klog_diff = fwts_klog_find_changes(klog_pre, klog_post);
+	s4_check_log(fw, klog_diff, klog_errors, klog_oopses, klog_warn_ons);
 
 	fwts_progress_message(fw, percent, "(Checking for PM errors)");
 
@@ -180,60 +177,54 @@ static int s4_hibernate(fwts_framework *fw,
 		fwts_failed(fw, LOG_LEVEL_HIGH, "PMActionFailedPreS4",
 			"pm-action failed before trying to put the system "
 			"in the requested power saving state.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 		(*pm_errors)++;
 	} else if (status == 128) {
 		fwts_failed(fw, LOG_LEVEL_HIGH, "PMActionPowerStateS4",
 			"pm-action tried to put the machine in the requested "
 			"power state but failed.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 		(*pm_errors)++;
 	} else if (status > 128) {
 		fwts_failed(fw, LOG_LEVEL_HIGH, "PMActionFailedS4",
 			"pm-action encountered an error and also failed to "
 			"enter the requested power saving state.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 		(*pm_errors)++;
 	}
 
-	if (fwts_klog_regex_find(fw, klog, "Freezing user space processes.*done") < 1) {
+	if (fwts_klog_regex_find(fw, klog_diff, "Freezing user space processes.*done") < 1) {
 		fwts_failed(fw, LOG_LEVEL_HIGH, "UserSpaceTaskFreeze",
 			"Failed to freeze user space processes.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 		(*pm_errors)++;
 	}
 
-	if (fwts_klog_regex_find(fw, klog, "Freezing remaining freezable tasks.*done") < 1) {
+	if (fwts_klog_regex_find(fw, klog_diff, "Freezing remaining freezable tasks.*done") < 1) {
 		fwts_failed(fw, LOG_LEVEL_HIGH, "KernelTaskFreeze",
 			"Failed to freeze remaining non-user space processes.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 		(*pm_errors)++;
 	}
 
-	if ((fwts_klog_regex_find(fw, klog, "PM: freeze of devices complete") < 1) &&
-	    (fwts_klog_regex_find(fw, klog, "PM: late freeze of devices complete") < 1)) {
+	if ((fwts_klog_regex_find(fw, klog_diff, "PM: freeze of devices complete") < 1) &&
+	    (fwts_klog_regex_find(fw, klog_diff, "PM: late freeze of devices complete") < 1)) {
 		fwts_failed(fw, LOG_LEVEL_HIGH, "DeviceFreeze",
 			"Failed to freeze devices.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 		(*pm_errors)++;
 	}
 
-	if (fwts_klog_regex_find(fw, klog, "PM: Allocated.*kbytes") < 1) {
+	if (fwts_klog_regex_find(fw, klog_diff, "PM: Allocated.*kbytes") < 1) {
 		fwts_failed(fw, LOG_LEVEL_HIGH, "HibernateImageAlloc",
 			"Failed to allocate memory for hibernate image.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 		*failed_alloc_image = 1;
 		(*pm_errors)++;
 	}
 
-	if (fwts_klog_regex_find(fw, klog, "PM: Image restored successfully") < 1) {
+	if (fwts_klog_regex_find(fw, klog_diff, "PM: Image restored successfully") < 1) {
 		fwts_failed(fw, LOG_LEVEL_HIGH, "HibernateImageRestore",
 			"Failed to restore hibernate image.");
-		fwts_tag_failed(fw, FWTS_TAG_POWER_MANAGEMENT);
 		(*pm_errors)++;
 	}
 
-	fwts_klog_free(klog);
+	fwts_klog_free(klog_pre);
+	fwts_klog_free(klog_post);
+	fwts_list_free(klog_diff, NULL);
 
 	return FWTS_OK;
 }
