@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,8 +18,6 @@
  */
 #include "fwts.h"
 
-#ifdef FWTS_ARCH_INTEL
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -211,6 +209,16 @@ static syntaxcheck_error_map_item syntaxcheck_error_map[] = {
     ASL_ID(ASL_MSG_UNKNOWN_SUBTABLE),
     ASL_ID(ASL_MSG_UNKNOWN_TABLE),
     ASL_ID(ASL_MSG_ZERO_VALUE),
+    ASL_ID(ASL_MSG_RECURSION),
+    ASL_ID(ASL_MSG_SERIALIZED_REQUIRED),
+    ASL_ID(ASL_MSG_LOCAL_OUTSIDE_METHOD),
+    ASL_ID(ASL_MSG_NOT_PARAMETER),
+    ASL_ID(ASL_MSG_TRUNCATION),
+    ASL_ID(ASL_MSG_LIST_LENGTH_SHORT),
+    ASL_ID(ASL_MSG_PACKAGE_LENGTH),
+    ASL_ID(ASL_MSG_RESERVED_PACKAGE_LENGTH),
+    ASL_ID(ASL_MSG_LIST_LENGTH_SHORT),
+
     { 0, NULL, NULL }
 };
 
@@ -429,9 +437,9 @@ static int syntaxcheck_table(fwts_framework *fw, char *tablename, int which)
 	fwts_list_link *item;
 	int errors = 0;
 	int warnings = 0;
+	int remarks = 0;
 	fwts_acpi_table_info *table;
-	fwts_list* iasl_errors;
-	fwts_list* iasl_disassembly;
+	fwts_list *iasl_stdout, *iasl_stderr, *iasl_disassembly;
 
 	if (fwts_acpi_find_table(fw, tablename, which, &table) != FWTS_OK) {
 		fwts_aborted(fw, "Cannot load ACPI table %s.", tablename);
@@ -442,7 +450,7 @@ static int syntaxcheck_table(fwts_framework *fw, char *tablename, int which)
 		return FWTS_NO_TABLE;		/* Table does not exist */
 
 	if (fwts_iasl_reassemble(fw, table->data, table->length,
-				&iasl_disassembly, &iasl_errors) != FWTS_OK) {
+				&iasl_disassembly, &iasl_stdout, &iasl_stderr) != FWTS_OK) {
 		fwts_aborted(fw, "Cannot re-assasemble with iasl.");
 		return FWTS_ERROR;
 	}
@@ -451,9 +459,9 @@ static int syntaxcheck_table(fwts_framework *fw, char *tablename, int which)
 	fwts_log_info(fw, "Checking ACPI table %s (#%d)", tablename, which);
 	fwts_log_nl(fw);
 
-	if (iasl_errors) {
+	if (iasl_stdout) {
 		/* Scan error text from assembly */
-		fwts_list_foreach(item, iasl_errors) {
+		fwts_list_foreach(item, iasl_stdout) {
 			int num;
 			char ch;
 			char *line = fwts_text_list_text(item);
@@ -463,12 +471,13 @@ static int syntaxcheck_table(fwts_framework *fw, char *tablename, int which)
 					char *error_text = fwts_text_list_text(item->next);
 					int iasl_error = (strstr(error_text, "Error") != NULL);
 					int iasl_warning = (strstr(error_text, "Warning") != NULL);
+					int iasl_remark = (strstr(error_text, "Remark") != NULL);
 					int error_code;
 
 					sscanf(error_text, "%*s %d", &error_code);
 
 					/* Valid error or warning, go and report */
-					if (iasl_error || iasl_warning) {
+					if (iasl_error || iasl_warning || iasl_remark) {
 						char label[64];
 						char *colon = strstr(line, ":");
 						char *carat = strstr(error_text, "^");
@@ -511,7 +520,7 @@ static int syntaxcheck_table(fwts_framework *fw, char *tablename, int which)
 							fwts_failed(fw, LOG_LEVEL_HIGH, label, "Assembler error in line %d", num);
 							break;
 						case ASL_REMARK:
-							fwts_log_info(fw, "Assembler remark in line %d", num);
+							fwts_failed(fw, LOG_LEVEL_LOW, label, "Assembler remark in line %d", num);
 							break;
 						case ASL_OPTIMIZATION:
 							skip = true;
@@ -530,6 +539,7 @@ static int syntaxcheck_table(fwts_framework *fw, char *tablename, int which)
 					}
 					errors += iasl_error;
 					warnings += iasl_warning;
+					remarks += iasl_remark;
 					item = item->next;
 				}
 				else {
@@ -541,15 +551,34 @@ static int syntaxcheck_table(fwts_framework *fw, char *tablename, int which)
 			}
 		}
 	}
-	fwts_text_list_free(iasl_disassembly);
-	fwts_text_list_free(iasl_errors);
 
-	if (errors > 0) {
-		fwts_log_info(fw, "Table %s (%d) reassembly: Found %d errors, %d warnings.", tablename, which, errors, warnings);
-	} else if (warnings > 0) {
-		fwts_log_info(fw, "Table %s (%d) reassembly: Found 0 errors, %d warnings.", tablename, which, warnings);
-	} else
-		fwts_passed(fw, "%s (%d) reassembly, Found 0 errors, 0 warnings.", tablename, which);
+	/*
+	 *  Did iasl emit any errors that we need to check?
+	 */
+	if (iasl_stderr) {
+		fwts_list_foreach(item, iasl_stderr) {
+			char *line = fwts_text_list_text(item);
+			if (strstr(line, "Compiler aborting due to parser-detected syntax error")) {
+				fwts_failed(fw, LOG_LEVEL_HIGH, "SyntaxCheckIASLCompilerAborted",
+					"Compilation aborted early due to a parser detected syntax error.");
+				fwts_advice(fw,
+					"Some subsequent errors may not be detected because the "
+					"compiler had to terminate prematurely.  If the compiler did not "
+					"abort early then potentially correct code may parse incorrectly "
+					"producing some or many false positive errors.");
+			}
+		}
+	}
+
+	fwts_text_list_free(iasl_disassembly);
+	fwts_text_list_free(iasl_stdout);
+	fwts_text_list_free(iasl_stderr);
+
+	if (errors + warnings + remarks > 0)
+		fwts_log_info(fw, "Table %s (%d) reassembly: Found %d errors, %d warnings, %d remarks.",
+			tablename, which, errors, warnings, remarks);
+	else
+		fwts_passed(fw, "%s (%d) reassembly, Found 0 errors, 0 warnings, 0 remarks.", tablename, which);
 
 	fwts_log_nl(fw);
 
@@ -590,5 +619,3 @@ static fwts_framework_ops syntaxcheck_ops = {
 };
 
 FWTS_REGISTER("syntaxcheck", &syntaxcheck_ops, FWTS_TEST_ANYTIME, FWTS_FLAG_BATCH);
-
-#endif
