@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,27 +25,23 @@
 #include <unistd.h>
 #include <string.h>
 
-static const char *wkalarm = WAKEALARM;
-
 static int wakealarm_test1(fwts_framework *fw)
 {
-	struct stat buf;
-
-	if (stat(wkalarm, &buf) == 0)
-		fwts_passed(fw, WAKEALARM " found.");
+	if (fwts_wakealarm_exits(fw) == FWTS_OK)
+		fwts_passed(fw, "RTC with a RTC alarm ioctl() interface found.");
 	else {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "NoWakeAlarmTest1",
-			"Could not find " WAKEALARM ".");
+			"Could not find an RTC with an alarm ioctl() interface.");
 #ifdef FWTS_ARCH_INTEL
 		/* For x86 devices, this is considered a failure */
 		fwts_advice(fw,
 			"x86 devices generally should have an RTC wake alarm that "
-			"is normally controlled by the " WAKEALARM " interface. This interface "
+			"is normally controlled by the RTC alarm ioctl() interface. This interface "
 			"does not exist, so the wake alarm tests will be aborted.");
 #else
 		fwts_advice(fw,
 			"non-x86 devices sometimes do not have an RTC wake alarm that "
-			"is normally controlled by the " WAKEALARM " interface. This "
+			"is normally controlled by the RTC alarm ioctl() interface. This "
 			"interface does not exist, so the wake alarm tests will be aborted.");
 #endif
 		return FWTS_ABORTED;
@@ -60,10 +56,11 @@ static int wakealarm_test2(fwts_framework *fw)
 	if (fwts_wakealarm_trigger(fw, 1)) {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "WakeAlarmNotTriggeredTest2",
 			"RTC wakealarm did not trigger.");
-		fwts_tag_failed(fw, FWTS_TAG_BIOS);
 		return FWTS_OK;
 	}
 
+	(void)fwts_wakealarm_cancel(fw);
+
 	fwts_passed(fw, "RTC wakealarm was triggered successfully.");
 
 	return FWTS_OK;
@@ -77,7 +74,6 @@ static int wakealarm_test3(fwts_framework *fw)
 	if (ret < 0) {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "WakeAlarmNotTriggeredTest3",
 			"Failed to trigger and fire wakealarm.");
-		fwts_tag_failed(fw, FWTS_TAG_BIOS);
 		return FWTS_ERROR;	/* Really went wrong */
 	}
 	if (ret == 0)
@@ -85,7 +81,6 @@ static int wakealarm_test3(fwts_framework *fw)
 	else {
 		fwts_failed(fw, LOG_LEVEL_MEDIUM, "WakeAlarmNotFiredTest3",
 			"RTC wakealarm was triggered but did not fire.");
-		fwts_tag_failed(fw, FWTS_TAG_BIOS);
 	}
 
 	return FWTS_OK;
@@ -103,14 +98,12 @@ static int wakealarm_test4(fwts_framework *fw)
 			fwts_failed(fw, LOG_LEVEL_MEDIUM,
 				"WakeAlarmNotTriggeredTest4",
 				"Failed to trigger and fire wakealarm.");
-			fwts_tag_failed(fw, FWTS_TAG_BIOS);
 			return FWTS_ERROR;	/* Really went wrong */
 		}
 		if (ret != 0) {
 			fwts_failed(fw, LOG_LEVEL_MEDIUM,
 				"WakeAlarmNotFiredTest4",
 				"RTC wakealarm was triggered but did not fire.");
-			fwts_tag_failed(fw, FWTS_TAG_BIOS);
 			failed++;
 		}
 		fwts_progress(fw, 25 * i);
@@ -122,15 +115,15 @@ static int wakealarm_test4(fwts_framework *fw)
 }
 
 static fwts_framework_minor_test wakealarm_tests[] = {
-	{ wakealarm_test1, "Check existence of " WAKEALARM "." },
+	{ wakealarm_test1, "Test existence of RTC with alarm interface." },
 	{ wakealarm_test2, "Trigger wakealarm for 1 seconds in the future." },
-	{ wakealarm_test3, "Check if wakealarm is fired." },
+	{ wakealarm_test3, "Test if wakealarm is fired." },
 	{ wakealarm_test4, "Multiple wakealarm firing tests." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops wakealarm_ops = {
-	.description = "Test ACPI Wakealarm.",
+	.description = "ACPI Wakealarm tests.",
 	.minor_tests = wakealarm_tests
 };
 
