@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -427,12 +427,17 @@ void fwts_acpica_vprintf(const char *fmt, va_list args)
 		else
 			buffer_len = 0;
 	} else {
+		char *new_buf;
+
 		buffer_len += tmp_len;
-		buffer = realloc(buffer, buffer_len);
-		if (buffer)
+		new_buf = realloc(buffer, buffer_len);
+		if (new_buf) {
+			buffer = new_buf;
 			strcat(buffer, tmp);
-		else
+		} else {
+			free(buffer);
 			buffer_len = 0;
+		}
 	}
 
 	if (index(buffer, '\n') != NULL) {
@@ -903,7 +908,7 @@ int fwts_acpica_init(fwts_framework *fw)
 	if (fwts_acpica_init_called)
 		return FWTS_ERROR;
 
-	AcpiGbl_AllMethodsSerialized =
+	AcpiGbl_AutoSerializeMethods =
 		FWTS_ACPICA_MODE(fw, FWTS_ACPICA_MODE_SERIALIZED);
 	AcpiGbl_EnableInterpreterSlack =
 		FWTS_ACPICA_MODE(fw, FWTS_ACPICA_MODE_SLACK);
@@ -922,11 +927,6 @@ int fwts_acpica_init(fwts_framework *fw)
 
 	AcpiOsRedirectOutput(stderr);
 
-	if (ACPI_FAILURE(AcpiInitializeSubsystem())) {
-		fwts_log_error(fw, "Failed to initialise ACPICA subsystem.");
-		return FWTS_ERROR;
-	}
-
 	/* Clone FADT, make sure it points to a DSDT in user space */
 	if (fwts_acpi_find_table(fw, "FACP", 0, &table) != FWTS_OK)
 		return FWTS_ERROR;
@@ -1061,27 +1061,34 @@ int fwts_acpica_init(fwts_framework *fw)
 		fwts_acpica_RSDP = NULL;
 	}
 
+	if (ACPI_FAILURE(AcpiInitializeSubsystem())) {
+		fwts_log_error(fw, "Failed to initialise ACPICA subsystem.");
+		goto failed;
+	}
+
 	if (AcpiInitializeTables(Tables, ACPI_MAX_INIT_TABLES, TRUE) != AE_OK) {
 		fwts_log_error(fw, "Failed to initialise tables.");
-		return FWTS_ERROR;
+		goto failed;
 	}
 	if (AcpiReallocateRootTable() != AE_OK) {
 		fwts_log_error(fw, "Failed to reallocate root table.");
-		return FWTS_ERROR;
+		goto failed;
 	}
 	if (AcpiLoadTables() != AE_OK) {
 		fwts_log_error(fw, "Failed to load tables.");
-		return FWTS_ERROR;
+		goto failed;
 	}
 
-
 	(void)fwtsInstallEarlyHandlers(fw);
 	AcpiEnableSubsystem(init_flags);
 	AcpiInitializeObjects(init_flags);
 
 	fwts_acpica_init_called = true;
-
 	return FWTS_OK;
+
+failed:
+	AcpiTerminate();
+	return FWTS_ERROR;
 }
 
 #define FWTS_ACPICA_FREE(x)	\
