@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2013, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2014, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -383,21 +383,37 @@ DtCompileInteger (
         return;
     }
 
-    /* Ensure that reserved fields are set to zero */
-    /* TBD: should we set to zero, or just make this an ERROR? */
-    /* TBD: Probably better to use a flag */
+    /*
+     * Ensure that reserved fields are set properly. Note: uses
+     * the DT_NON_ZERO flag to indicate that the reserved value
+     * must be exactly one. Otherwise, the value must be zero.
+     * This is sufficient for now.
+     */
 
-    if (!ACPI_STRCMP (Field->Name, "Reserved") &&
-        (Value != 0))
+    /* TBD: Should use a flag rather than compare "Reserved" */
+
+    if (!ACPI_STRCMP (Field->Name, "Reserved"))
     {
-        DtError (ASL_WARNING, ASL_MSG_RESERVED_VALUE, Field,
-            "Setting to zero");
-        Value = 0;
+        if (Flags & DT_NON_ZERO)
+        {
+            if (Value != 1)
+            {
+                DtError (ASL_WARNING, ASL_MSG_RESERVED_VALUE, Field,
+                    "Must be one, setting to one");
+                Value = 1;
+            }
+        }
+        else if (Value != 0)
+        {
+            DtError (ASL_WARNING, ASL_MSG_RESERVED_VALUE, Field,
+                "Must be zero, setting to zero");
+            Value = 0;
+        }
     }
 
     /* Check if the value must be non-zero */
 
-    if ((Value == 0) && (Flags & DT_NON_ZERO))
+    else if ((Flags & DT_NON_ZERO) && (Value == 0))
     {
         DtError (ASL_ERROR, ASL_MSG_ZERO_VALUE, Field, NULL);
     }
@@ -412,7 +428,8 @@ DtCompileInteger (
 
     if (Value > MaxValue)
     {
-        sprintf (MsgBuffer, "%8.8X%8.8X", ACPI_FORMAT_UINT64 (Value));
+        sprintf (MsgBuffer, "%8.8X%8.8X - max %u bytes",
+            ACPI_FORMAT_UINT64 (Value), ByteLength);
         DtError (ASL_ERROR, ASL_MSG_INTEGER_SIZE, Field, MsgBuffer);
     }
 
