@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2013, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2014, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -356,36 +356,6 @@ DtStrtoul64 (
 }
 
 
-/******************************************************************************
- *
- * FUNCTION:    DtGetFileSize
- *
- * PARAMETERS:  Handle              - Open file handler
- *
- * RETURN:      Current file size
- *
- * DESCRIPTION: Get the current size of a file. Seek to the EOF and get the
- *              offset. Seek back to the original location.
- *
- *****************************************************************************/
-
-UINT32
-DtGetFileSize (
-    FILE                    *Handle)
-{
-    int                     CurrentOffset;
-    int                     LastOffset;
-
-
-    CurrentOffset = ftell (Handle);
-    fseek (Handle, 0, SEEK_END);
-    LastOffset = ftell (Handle);
-    fseek (Handle, CurrentOffset, SEEK_SET);
-
-    return ((UINT32) LastOffset);
-}
-
-
 /******************************************************************************
  *
  * FUNCTION:    DtGetFieldValue
@@ -604,6 +574,7 @@ DtGetFieldLength (
     case ACPI_DMT_ACCWIDTH:
     case ACPI_DMT_IVRS:
     case ACPI_DMT_MADT:
+    case ACPI_DMT_PCCT:
     case ACPI_DMT_PMTT:
     case ACPI_DMT_SRAT:
     case ACPI_DMT_ASF:
