@@ -21,6 +21,6 @@
 #define __FWTS_IASL_INTERFACE__
 
 int fwts_iasl_disassemble_aml(const char *aml, const char *outputfile);
-int fwts_iasl_assemble_aml(const char *source, char **output);
+int fwts_iasl_assemble_aml(const char *source, char **stdout_output, char **stderr_output);
 
 #endif
