@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2013, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2014, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -432,7 +432,7 @@ PrAddMacro (
         if (ArgCount >= PR_MAX_MACRO_ARGS)
         {
             PrError (ASL_ERROR, ASL_MSG_TOO_MANY_ARGUMENTS, TokenOffset);
-            return;
+            goto ErrorExit;
         }
     }
 
@@ -472,7 +472,7 @@ PrAddMacro (
                     PrError (ASL_ERROR, ASL_MSG_TOO_MANY_ARGUMENTS,
                         THIS_TOKEN_OFFSET (Token));
 
-                    return;
+                    goto ErrorExit;
                 }
                 break;
             }
@@ -504,7 +504,7 @@ AddMacroToList:
                 THIS_TOKEN_OFFSET (Name));
         }
 
-        return;
+        goto ErrorExit;
     }
 
     DbgPrint (ASL_DEBUG_OUTPUT, PR_PREFIX_ID
@@ -523,6 +523,13 @@ AddMacroToList:
         DefineInfo->Args = Args;
         DefineInfo->ArgCount = ArgCount;
     }
+
+    return;
+
+
+ErrorExit:
+    ACPI_FREE (Args);
+    return;
 }
 
 
