@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2013, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2014, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -170,6 +170,11 @@ AeAttachedDataHandler (
     ACPI_HANDLE             Object,
     void                    *Data);
 
+static void
+AeAttachedDataHandler2 (
+    ACPI_HANDLE             Object,
+    void                    *Data);
+
 static UINT32
 AeInterfaceHandler (
     ACPI_STRING             InterfaceName,
@@ -198,6 +203,10 @@ static UINT32
 AeEventHandler (
     void                    *Context);
 
+static UINT32
+AeSciHandler (
+    void                    *Context);
+
 static char                *TableEvents[] =
 {
     "LOAD",
@@ -212,18 +221,22 @@ static AE_DEBUG_REGIONS     AeRegions;
 BOOLEAN                     AcpiGbl_DisplayRegionAccess = FALSE;
 
 /*
- * We will override some of the default region handlers, especially the
- * SystemMemory handler, which must be implemented locally. Do not override
- * the PCI_Config handler since we would like to exercise the default handler
- * code. These handlers are installed "early" - before any _REG methods
+ * We will override some of the default region handlers, especially
+ * the SystemMemory handler, which must be implemented locally.
+ * These handlers are installed "early" - before any _REG methods
  * are executed - since they are special in the sense that the ACPI spec
  * declares that they must "always be available". Cannot override the
  * DataTable region handler either -- needed for test execution.
+ *
+ * NOTE: The local region handler will simulate access to these address
+ * spaces by creating a memory buffer behind each operation region.
  */
 static ACPI_ADR_SPACE_TYPE  DefaultSpaceIdList[] =
 {
     ACPI_ADR_SPACE_SYSTEM_MEMORY,
-    ACPI_ADR_SPACE_SYSTEM_IO
+    ACPI_ADR_SPACE_SYSTEM_IO,
+    ACPI_ADR_SPACE_PCI_CONFIG,
+    ACPI_ADR_SPACE_EC
 };
 
 /*
@@ -282,6 +295,7 @@ AeCtrlCHandler (
         }
     }
 
+    (void) AcpiOsTerminate ();
     exit (0);
 }
 
@@ -370,7 +384,7 @@ AeCommonNotifyHandler (
 
         printf ("[AcpiExec] Handler %u: Received a %s Notify on [%4.4s] %p Value 0x%2.2X (%s)\n",
             HandlerId, Type, AcpiUtGetNodeName (Device), Device, Value,
-            AcpiUtGetNotifyName (Value));
+            AcpiUtGetNotifyName (Value, ACPI_TYPE_ANY));
         if (AcpiGbl_DebugFile)
         {
             AcpiOsPrintf ("[AcpiExec] Handler %u: Received a %s notify, Value 0x%2.2X\n",
@@ -406,7 +420,7 @@ AeSystemNotifyHandler (
 
     printf ("[AcpiExec] Global:    Received a System Notify on [%4.4s] %p Value 0x%2.2X (%s)\n",
         AcpiUtGetNodeName (Device), Device, Value,
-        AcpiUtGetNotifyName (Value));
+        AcpiUtGetNotifyName (Value, ACPI_TYPE_ANY));
     if (AcpiGbl_DebugFile)
     {
         AcpiOsPrintf ("[AcpiExec] Global:    Received a System Notify, Value 0x%2.2X\n", Value);
@@ -439,7 +453,7 @@ AeDeviceNotifyHandler (
 
     printf ("[AcpiExec] Global:    Received a Device Notify on [%4.4s] %p Value 0x%2.2X (%s)\n",
         AcpiUtGetNodeName (Device), Device, Value,
-        AcpiUtGetNotifyName (Value));
+        AcpiUtGetNotifyName (Value, ACPI_TYPE_ANY));
     if (AcpiGbl_DebugFile)
     {
         AcpiOsPrintf ("[AcpiExec] Global:    Received a Device Notify, Value 0x%2.2X\n", Value);
@@ -522,6 +536,8 @@ AeExceptionHandler (
             NewAmlStatus = (ACPI_STATUS)
                 ((ACPI_OBJECT *) ReturnObj.Pointer)->Integer.Value;
 
+            /* Free a buffer created via ACPI_ALLOCATE_BUFFER */
+
             AcpiOsFree (ReturnObj.Pointer);
         }
     }
@@ -672,7 +688,29 @@ AeAttachedDataHandler (
     ACPI_NAMESPACE_NODE     *Node = ACPI_CAST_PTR (ACPI_NAMESPACE_NODE, Data);
 
 
-    AcpiOsPrintf ("Received an attached data deletion on %4.4s\n",
+    AcpiOsPrintf ("Received an attached data deletion (1) on %4.4s\n",
+        Node->Name.Ascii);
+}
+
+
+/******************************************************************************
+ *
+ * FUNCTION:    AeAttachedDataHandler2
+ *
+ * DESCRIPTION: Handler for deletion of nodes with attached data (attached via
+ *              AcpiAttachData)
+ *
+ *****************************************************************************/
+
+static void
+AeAttachedDataHandler2 (
+    ACPI_HANDLE             Object,
+    void                    *Data)
+{
+    ACPI_NAMESPACE_NODE     *Node = ACPI_CAST_PTR (ACPI_NAMESPACE_NODE, Data);
+
+
+    AcpiOsPrintf ("Received an attached data deletion (2) on %4.4s\n",
         Node->Name.Ascii);
 }
 
@@ -704,9 +742,9 @@ AeInterfaceHandler (
 #if (!ACPI_REDUCED_HARDWARE)
 /******************************************************************************
  *
- * FUNCTION:    AeEventHandler
+ * FUNCTION:    AeEventHandler, AeSciHandler
  *
- * DESCRIPTION: Handler for Fixed Events
+ * DESCRIPTION: Handler for Fixed Events and SCIs
  *
  *****************************************************************************/
 
@@ -716,6 +754,16 @@ AeEventHandler (
 {
     return (0);
 }
+
+static UINT32
+AeSciHandler (
+    void                    *Context)
+{
+
+    AcpiOsPrintf ("[AcpiExec] Received an SCI at handler\n");
+    return (0);
+}
+
 #endif /* !ACPI_REDUCED_HARDWARE */
 
 
@@ -738,15 +786,65 @@ AeRegionInit (
     void                        *HandlerContext,
     void                        **RegionContext)
 {
-    /*
-     * Real simple, set the RegionContext to the RegionHandle
-     */
-    *RegionContext = RegionHandle;
+
+    if (Function == ACPI_REGION_DEACTIVATE)
+    {
+        *RegionContext = NULL;
+    }
+    else
+    {
+        *RegionContext = RegionHandle;
+    }
 
     return (AE_OK);
 }
 
 
+/*******************************************************************************
+ *
+ * FUNCTION:    AeInstallSciHandler
+ *
+ * PARAMETERS:  None
+ *
+ * RETURN:      Status
+ *
+ * DESCRIPTION: Install handler for SCIs. Exercise the code by doing an
+ *              install/remove/install.
+ *
+ ******************************************************************************/
+
+static ACPI_STATUS
+AeInstallSciHandler (
+    void)
+{
+    ACPI_STATUS             Status;
+
+
+    Status = AcpiInstallSciHandler (AeSciHandler, &AeMyContext);
+    if (ACPI_FAILURE (Status))
+    {
+        ACPI_EXCEPTION ((AE_INFO, Status,
+            "Could not install an SCI handler (1)"));
+    }
+
+    Status = AcpiRemoveSciHandler (AeSciHandler);
+    if (ACPI_FAILURE (Status))
+    {
+        ACPI_EXCEPTION ((AE_INFO, Status,
+            "Could not remove an SCI handler"));
+    }
+
+    Status = AcpiInstallSciHandler (AeSciHandler, &AeMyContext);
+    if (ACPI_FAILURE (Status))
+    {
+        ACPI_EXCEPTION ((AE_INFO, Status,
+            "Could not install an SCI handler (2)"));
+    }
+
+    return (Status);
+}
+
+
 /*******************************************************************************
  *
  * FUNCTION:    AeInstallDeviceHandlers, AeInstallEcHandler,
@@ -757,7 +855,7 @@ AeRegionInit (
  * RETURN:      Status
  *
  * DESCRIPTION: Walk entire namespace, install a handler for every EC
- *              device found.
+ *              and PCI device found.
  *
  ******************************************************************************/
 
@@ -857,6 +955,11 @@ AeInstallLateHandlers (
 #if (!ACPI_REDUCED_HARDWARE)
     if (!AcpiGbl_ReducedHardware)
     {
+        /* Install a user SCI handler */
+
+        Status = AeInstallSciHandler ();
+        AE_CHECK_OK (AeInstallSciHandler, Status);
+
         /* Install some fixed event handlers */
 
         Status = AcpiInstallFixedEventHandler (ACPI_EVENT_GLOBAL, AeEventHandler, NULL);
@@ -1026,6 +1129,11 @@ AeInstallEarlyHandlers (
 
         Status = AcpiAttachData (Handle, AeAttachedDataHandler, Handle);
         AE_CHECK_OK (AcpiAttachData, Status);
+
+        /* Test support for multiple attaches */
+
+        Status = AcpiAttachData (Handle, AeAttachedDataHandler2, Handle);
+        AE_CHECK_OK (AcpiAttachData, Status);
     }
     else
     {
@@ -1120,13 +1228,20 @@ AeRegionHandler (
 
     ACPI_OPERAND_OBJECT     *RegionObject = ACPI_CAST_PTR (ACPI_OPERAND_OBJECT, RegionContext);
     UINT8                   *Buffer = ACPI_CAST_PTR (UINT8, Value);
+    UINT8                   *OldBuffer;
+    UINT8                   *NewBuffer;
     ACPI_PHYSICAL_ADDRESS   BaseAddress;
+    ACPI_PHYSICAL_ADDRESS   BaseAddressEnd;
+    ACPI_PHYSICAL_ADDRESS   RegionAddress;
+    ACPI_PHYSICAL_ADDRESS   RegionAddressEnd;
     ACPI_SIZE               Length;
     BOOLEAN                 BufferExists;
+    BOOLEAN                 BufferResize;
     AE_REGION               *RegionElement;
     void                    *BufferValue;
     ACPI_STATUS             Status;
     UINT32                  ByteWidth;
+    UINT32                  RegionLength;
     UINT32                  i;
     UINT8                   SpaceId;
     ACPI_CONNECTION_INFO    *MyContext;
@@ -1409,21 +1524,119 @@ AeRegionHandler (
      * Search through the linked list for this region's buffer
      */
     BufferExists = FALSE;
+    BufferResize = FALSE;
     RegionElement = AeRegions.RegionList;
 
     if (AeRegions.NumberOfRegions)
     {
+        BaseAddressEnd = BaseAddress + Length - 1;
         while (!BufferExists && RegionElement)
         {
-            if (RegionElement->Address == BaseAddress &&
-                RegionElement->Length == Length &&
-                RegionElement->SpaceId == SpaceId)
+            RegionAddress = RegionElement->Address;
+            RegionAddressEnd = RegionElement->Address + RegionElement->Length - 1;
+            RegionLength = RegionElement->Length;
+
+            /*
+             * Overlapping Region Support
+             *
+             * While searching through the region buffer list, determine if an
+             * overlap exists between the requested buffer space and the current
+             * RegionElement space. If there is an overlap then replace the old
+             * buffer with a new buffer of increased size before continuing to
+             * do the read or write
+             */
+            if (RegionElement->SpaceId != SpaceId ||
+                BaseAddressEnd < RegionAddress ||
+                BaseAddress > RegionAddressEnd)
             {
-                BufferExists = TRUE;
+                /*
+                 * Requested buffer is outside of the current RegionElement
+                 * bounds
+                 */
+                RegionElement = RegionElement->NextRegion;
             }
             else
             {
-                RegionElement = RegionElement->NextRegion;
+                /*
+                 * Some amount of buffer space sharing exists. There are 4 cases
+                 * to consider:
+                 *
+                 * 1. Right overlap
+                 * 2. Left overlap
+                 * 3. Left and right overlap
+                 * 4. Fully contained - no resizing required
+                 */
+                BufferExists = TRUE;
+
+                if ((BaseAddress >= RegionAddress) &&
+                    (BaseAddress <= RegionAddressEnd) &&
+                    (BaseAddressEnd > RegionAddressEnd))
+                {
+                    /* Right overlap */
+
+                    RegionElement->Length = BaseAddress -
+                        RegionAddress + Length;
+                    BufferResize = TRUE;
+                }
+
+                else if ((BaseAddressEnd >= RegionAddress) &&
+                         (BaseAddressEnd <= RegionAddressEnd) &&
+                         (BaseAddress < RegionAddress))
+                {
+                    /* Left overlap */
+
+                    RegionElement->Address = BaseAddress;
+                    RegionElement->Length = RegionAddress -
+                        BaseAddress + RegionElement->Length;
+                    BufferResize = TRUE;
+                }
+
+                else if ((BaseAddress < RegionAddress) &&
+                         (BaseAddressEnd > RegionAddressEnd))
+                {
+                    /* Left and right overlap */
+
+                    RegionElement->Address = BaseAddress;
+                    RegionElement->Length = Length;
+                    BufferResize = TRUE;
+                }
+
+                /*
+                 * only remaining case is fully contained for which we don't
+                 * need to do anything
+                 */
+                if (BufferResize)
+                {
+                    NewBuffer = AcpiOsAllocate (RegionElement->Length);
+                    if (!NewBuffer)
+                    {
+                        return (AE_NO_MEMORY);
+                    }
+
+                    OldBuffer = RegionElement->Buffer;
+                    RegionElement->Buffer = NewBuffer;
+                    NewBuffer = NULL;
+
+                    /* Initialize the region with the default fill value */
+
+                    ACPI_MEMSET (RegionElement->Buffer,
+                        AcpiGbl_RegionFillValue, RegionElement->Length);
+
+                    /*
+                     * Get BufferValue to point (within the new buffer) to the
+                     * base address of the old buffer
+                     */
+                    BufferValue = (UINT8 *) RegionElement->Buffer +
+                        (UINT64) RegionAddress -
+                        (UINT64) RegionElement->Address;
+
+                    /*
+                     * Copy the old buffer to its same location within the new
+                     * buffer
+                     */
+                    ACPI_MEMCPY (BufferValue, OldBuffer, RegionLength);
+                    AcpiOsFree (OldBuffer);
+                }
             }
         }
     }
@@ -1433,9 +1646,8 @@ AeRegionHandler (
      */
     if (!BufferExists)
     {
-        /*
-         * Do the memory allocations first
-         */
+        /* Do the memory allocations first */
+
         RegionElement = AcpiOsAllocate (sizeof (AE_REGION));
         if (!RegionElement)
         {
@@ -1460,8 +1672,8 @@ AeRegionHandler (
 
         /*
          * Increment the number of regions and put this one
-         *  at the head of the list as it will probably get accessed
-         *  more often anyway.
+         * at the head of the list as it will probably get accessed
+         * more often anyway.
          */
         AeRegions.NumberOfRegions += 1;
 
@@ -1473,9 +1685,8 @@ AeRegionHandler (
         AeRegions.RegionList = RegionElement;
     }
 
-    /*
-     * Calculate the size of the memory copy
-     */
+    /* Calculate the size of the memory copy */
+
     ByteWidth = (BitWidth / 8);
 
     if (BitWidth % 8)
