@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006, Intel Corporation
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This file was originally part of the Linux-ready Firmware Developer Kit
  *
@@ -76,26 +76,20 @@ static int apicedge_test1(fwts_framework *fw)
 			edge = 0;
 
 		if (strstr(line,"acpi")) {
-			if (edge == 1) {
+			if (edge == 1)
 				fwts_failed(fw, LOG_LEVEL_MEDIUM,
 					"ACPIIRQEdgeTrig",
 					"ACPI Interrupt is incorrectly edge triggered.");
-				fwts_tag_failed(fw, FWTS_TAG_BIOS_IRQ);
-			}
 			continue;
 		}
-		if ((irq < 15) && (edge == 0)) {
+		if ((irq < 15) && (edge == 0))
 			fwts_failed(fw, LOG_LEVEL_MEDIUM,
 				"LegacyIRQLevelTrig",
 				"Legacy interrupt %i is incorrectly level triggered.", irq);
-			fwts_tag_failed(fw, FWTS_TAG_BIOS_IRQ);
-		}
-		if ((irq < 15) && (edge == -1)) {
+		if ((irq < 15) && (edge == -1))
 			fwts_failed(fw, LOG_LEVEL_MEDIUM,
 				"NonLegacyIRQLevelTrig",
 				"Non-Legacy interrupt %i is incorrectly level triggered.", irq);
-			fwts_tag_failed(fw, FWTS_TAG_BIOS_IRQ);
-		}
 	}
 	fclose(file);
 
@@ -106,12 +100,12 @@ static int apicedge_test1(fwts_framework *fw)
 }
 
 static fwts_framework_minor_test apicedge_tests[] = {
-	{ apicedge_test1, "Legacy and PCI Interrupt Edge/Level trigger checks." },
+	{ apicedge_test1, "Legacy and PCI Interrupt Edge/Level trigger tests." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops apicedge_ops = {
-	.description = "APIC Edge/Level Check.",
+	.description = "APIC edge/level test.",
 	.minor_tests = apicedge_tests
 };
 
