@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -152,8 +152,10 @@ static int hda_audio_test1(fwts_framework *fw)
 	struct dirent *directory;
 	bool checked = false;
 
-	if ((dir = opendir("/sys/class/sound/")) == NULL)
-		return FWTS_ERROR;
+	if ((dir = opendir("/sys/class/sound")) == NULL) {
+		fwts_skipped(fw, "Cannot find any sound devices in /sys/class/sound.");
+		return FWTS_OK;
+	}
 
 	while ((directory = readdir(dir)) != NULL)
 		if (strncmp(directory->d_name, "hw", 2) == 0) {
@@ -176,12 +178,12 @@ static int hda_audio_test1(fwts_framework *fw)
 }
 
 static fwts_framework_minor_test hda_audio_tests[] = {
-	{ hda_audio_test1, "Check HDA Audio Pin Configs." },
+	{ hda_audio_test1, "HDA Audio Pin Configuration test." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops hda_audio_ops = {
-	.description = "Check HDA Audio Pin Configs.",
+	.description = "HDA Audio Pin Configuration test.",
 	.minor_tests = hda_audio_tests
 };
 
