@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006, Intel Corporation
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This code was originally part of the Linux-ready Firmware Developer Kit
  *
@@ -201,35 +201,71 @@ static int check_prefetchable(
 	uint64_t address,
 	bool *pref)
 {
-	char line[4096];
-	fwts_list *lspci_output;
-	fwts_list_link *item;
-	int status;
+	char path[PATH_MAX];
+	uint8_t config[64];
+	int fd, i, n, bars;
+	uint32_t *bar;
 
 	*pref = false;
-
-	memset(line,0,4096);
-
-	snprintf(line, sizeof(line), "%s -v -s %s", fw->lspci, device);
-	if (fwts_pipe_exec(line, &lspci_output, &status) != FWTS_OK)
+	snprintf(path, sizeof(path), FWTS_PCI_DEV_PATH "/%s/config", device);
+	if ((fd = open(path, O_RDONLY)) < 0) {
+		fwts_log_error(fw, "Cannot read PCI config for device %s\n", device);
 		return FWTS_ERROR;
+	}
 
-	/* Nothing for this decice? That seems like a failure */
-	if (lspci_output == NULL)
-		return FWTS_ERROR;
+	n = read(fd, config, sizeof(config));
+	close(fd);
 
-	fwts_list_foreach(item, lspci_output) {
-		char *str = strstr(fwts_text_list_text(item), "Memory at ");
-		if (str && strtoull(str+10, NULL, 16) == address) {
-			if (strstr(str, "non-prefetchable"))
-				*pref = false;
-			else if (strstr(str, "(prefetchable"))
-				*pref = true;
-			else if (strstr(str, ", prefetchable"))
-				*pref = true;
+	/* config space too small? ignore for now */
+	if (n < 64)
+		return FWTS_OK;
+
+	/* Type, mask off top bit so we can cater for multi-function devices */
+	switch (config[FWTS_PCI_CONFIG_HEADER_TYPE] & 0x7f) {
+	case FWTS_PCI_CONFIG_HEADER_TYPE_NON_BRIDGE:
+		bars = 6;
+		break;
+	case FWTS_PCI_CONFIG_HEADER_TYPE_PCI_BRIDGE:
+		bars = 2;
+		break;
+	default:
+		/* No BARs, ignore */
+		return FWTS_OK;
+	}
+
+	/*
+	 *  Check BAR addresses, do they match and are they prefetchable
+	 *  See http://wiki.osdev.org/PCI
+	 */
+	bar = (uint32_t*)&config[0x10];
+	for (i = 0; i < bars; i++) {
+		if ((bar[i] & 1) == 0) {
+			uint64_t bar_addr;
+			bool is_prefetchable =
+				(bar[i] & FWTS_PCI_CONFIG_BAR_PREFETCHABLE);
+
+			switch ((bar[i] >> 1) & 0x03) {
+			case FWTS_PCI_CONFIG_BAR_TYPE_ADDR_16BIT:
+				bar_addr = bar[i] & 0xfff0;
+				break;
+			case FWTS_PCI_CONFIG_BAR_TYPE_ADDR_32BIT:
+				bar_addr = bar[i] & 0xfffffff0;
+				break;
+			case FWTS_PCI_CONFIG_BAR_TYPE_ADDR_64BIT:
+				bar_addr = (bar[i] & 0xfffffff0) |
+					   (((uint64_t)bar[i+1]) << 32);
+				i++;
+				break;
+			default:
+				continue;
+			}
+
+			if (bar_addr == address) {
+				*pref = is_prefetchable;
+				break;
+			}
 		}
 	}
-	fwts_list_free(lspci_output, free);
 
 	return FWTS_OK;
 }
@@ -334,8 +370,8 @@ static int validate_iomem(fwts_framework *fw)
 
 		if (guess_cache_type(fw, c2, &type_must, &type_mustnot, start) != FWTS_OK) {
 			/*  This has failed, give up at this point */
-			fwts_skipped(fw, 
-				"Could not guess cache type, lspci seems to be failing.");
+			fwts_skipped(fw,
+				"Could not guess cache type.");
 			fclose(file);
 			return FWTS_ERROR;
 		}
@@ -348,7 +384,6 @@ static int validate_iomem(fwts_framework *fw)
 				"has incorrect attribute%s.",
 				start, end,
 				c2, cache_to_string(type & type_mustnot));
-			fwts_tag_failed(fw, FWTS_TAG_BIOS);
 			if (type_must == UNCACHED)
 				skiperror = true;
 		}
@@ -366,7 +401,6 @@ static int validate_iomem(fwts_framework *fw)
 				start, end,
 				c2,
 				cache_to_string( (type & type_must) ^ type_must));
-			fwts_tag_failed(fw, FWTS_TAG_BIOS);
 		}
 
 	}
@@ -404,9 +438,6 @@ static void do_mtrr_resource(fwts_framework *fw)
 
 static int mtrr_init(fwts_framework *fw)
 {
-	if (fwts_check_executable(fw, fw->lspci, "lspci"))
-		return FWTS_ERROR;
-
 	if (get_mtrrs() != FWTS_OK) {
 		fwts_log_error(fw, "Failed to read /proc/mtrr.");
 		return FWTS_ERROR;
@@ -454,17 +485,14 @@ static int mtrr_test2(fwts_framework *fw)
 
 		if (fwts_klog_regex_find(fw, klog, "mtrr: your CPUs had inconsistent fixed MTRR settings") > 0) {
 			fwts_log_info(fw, "Detected CPUs with inconsistent fixed MTRR settings which the kernel fixed.");
-			fwts_tag_failed(fw, FWTS_TAG_BIOS);
 			failed = true;
 		}
 		if (fwts_klog_regex_find(fw, klog, "mtrr: your CPUs had inconsistent variable MTRR settings") > 0) {
 			fwts_log_info(fw, "Detected CPUs with inconsistent variable MTRR settings which the kernel fixed.");
-			fwts_tag_failed(fw, FWTS_TAG_BIOS);
 			failed = true;
 		}
 		if (fwts_klog_regex_find(fw, klog, "mtrr: your CPUs had inconsistent MTRRdefType") > 0) {
 			fwts_log_info(fw, "Detected CPUs with inconsistent variable MTRR settings which the kernel fixed.");
-			fwts_tag_failed(fw, FWTS_TAG_BIOS);
 			failed = true;
 		}
 
@@ -486,7 +514,7 @@ static int mtrr_test3(fwts_framework *fw)
 {
 	if (strstr(fwts_cpuinfo->vendor_id, "AMD")) {
 		if (klog != NULL) {
-			if (fwts_klog_regex_find(fw, klog, "SYSCFG[MtrrFixDramModEn] not cleared by BIOS, clearing this bit") > 0) {
+			if (fwts_klog_regex_find(fw, klog, "SYSCFG[MtrrFixDramModEn] not cleared by BIOS, clearing this bit") > 0)
 				fwts_failed(fw, LOG_LEVEL_MEDIUM,
 					"MTRRFixDramModEnBit",
 					"The BIOS is expected to clear MtrrFixDramModEn bit, see for example "
@@ -495,11 +523,8 @@ static int mtrr_test3(fwts_framework *fw)
  					"\"13.2.1.2 SYSCFG Register\": \"The MtrrFixDramModEn bit should be set "
  					"to 1 during BIOS initialization of the fixed MTRRs, then cleared to "
  					"0 for operation.\"");
-				fwts_tag_failed(fw, FWTS_TAG_BIOS);
-			}
-			else {
+			else
 				fwts_passed(fw, "No MtrrFixDramModEn error detected.");
-			}
 		}
 	} else
 		fwts_skipped(fw, "CPU is not an AMD, cannot test.");
@@ -510,12 +535,12 @@ static int mtrr_test3(fwts_framework *fw)
 static fwts_framework_minor_test mtrr_tests[] = {
 	{ mtrr_test1, "Validate the kernel MTRR IOMEM setup." },
 	{ mtrr_test2, "Validate the MTRR setup across all processors." },
-	{ mtrr_test3, "Check for AMD MtrrFixDramModEn being cleared by the BIOS." },
+	{ mtrr_test3, "Test for AMD MtrrFixDramModEn being cleared by the BIOS." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops mtrr_ops = {
-	.description = "MTRR validation.",
+	.description = "MTRR tests.",
 	.init        = mtrr_init,
 	.deinit      = mtrr_deinit,
 	.minor_tests = mtrr_tests
