@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006, Intel Corporation
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This code was originally part of the Linux-ready Firmware Developer Kit
  *
@@ -57,7 +57,6 @@ static int os2gap_test1(fwts_framework *fw)
 			OS2_GAP_SIZE / 1024,
 			(void*)OS2_GAP_ADDRESS,
 			(void*)(OS2_GAP_ADDRESS + OS2_GAP_SIZE));
-		fwts_tag_failed(fw, FWTS_TAG_BIOS);
 		fwts_log_nl(fw);
 		fwts_memory_map_table_dump(fw, os2gap_memory_map_info);
 	} else
@@ -67,7 +66,7 @@ static int os2gap_test1(fwts_framework *fw)
 }
 
 static fwts_framework_minor_test os2gap_tests[] = {
-	{ os2gap_test1, "Check the OS/2 15Mb memory hole is absent." },
+	{ os2gap_test1, "Test the OS/2 15Mb memory hole is absent." },
 	{ NULL, NULL }
 };
 
