@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2013 Canonical
+ * Copyright (C) 2011-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -167,12 +167,10 @@ static int pnp_test1(fwts_framework *fw)
 		fwts_log_info(fw,
 			"Could not find PnP BIOS Support Installation Check structure. "
 			"This is not necessarily a failure.");
-	else if (found > 1) {
+	else if (found > 1)
 		fwts_failed(fw, LOG_LEVEL_HIGH,
 			"PNPMultipleTables",
 			"Found %d instances of PCI Routing Tables, there should only be 1.", found);
-		fwts_tag_failed(fw, FWTS_TAG_BIOS);
-	}
 
         (void)fwts_munmap(mem, PNP_REGION_SIZE);
 
@@ -180,12 +178,12 @@ static int pnp_test1(fwts_framework *fw)
 }
 
 static fwts_framework_minor_test pnp_tests[] = {
-	{ pnp_test1, "Check PnP BIOS Support Installation structure." },
+	{ pnp_test1, "PnP BIOS Support Installation structure test." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops pnp_ops = {
-	.description = "Check BIOS Support Installation structure.",
+	.description = "BIOS Support Installation structure test.",
 	.minor_tests = pnp_tests
 };
 
