@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006, Intel Corporation
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This file was originally part of the Linux-ready Firmware Developer Kit
  *
@@ -26,11 +26,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdbool.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <limits.h>
 #include <dirent.h>
 #include <stdint.h>
+#include <stdbool.h>
 #include <sched.h>
 #include <time.h>
 #include <math.h>
@@ -49,7 +51,7 @@ typedef struct {
 static int number_of_speeds = -1;
 static int total_tests = 1;
 static int performed_tests = 0;
-static int no_cpufreq = 0;
+static bool no_cpufreq = false;
 static uint64_t top_speed = 0;
 static int num_cpus;
 
@@ -57,28 +59,15 @@ static int num_cpus;
 #define GET_PERFORMANCE_MIN (1)
 #define GET_PERFORMANCE_AVG (2)
 
-static void cpu_mkpath(char *path, const int len,
-	const int cpu, const char *name)
+static inline void cpu_mkpath(
+	char *const path,
+	const int len,
+	const int cpu,
+	const char *const name)
 {
 	snprintf(path, len, "%s/cpu%i/cpufreq/%s", FWTS_CPU_PATH, cpu, name);
 }
 
-static int count_ints(char *text)
-{
-	char *str = text;
-	int count = 0;
-
-	while (str && strlen(str) > 0) {
-		char *str2 = strchr(str, ' ');
-		if (!str2)
-			break;
-		str = str2 + 1;
-		count++;
-	}
-
-	return count;
-}
-
 static void set_governor(fwts_framework *fw, const int cpu)
 {
 	char path[PATH_MAX];
@@ -87,15 +76,15 @@ static void set_governor(fwts_framework *fw, const int cpu)
 
 	if (fwts_set("userspace", path) != FWTS_OK) {
 		if (!no_cpufreq) {
-			fwts_log_warning(fw,
-				"Frequency scaling not supported.");
-			no_cpufreq = 1;
+			fwts_warning(fw,
+				"Cannot set CPU scaling governor to userspace scaling.");
+			no_cpufreq = true;
 		}
 	}
 }
 
 #ifdef FWTS_ARCH_INTEL
-static int cpu_exists(int cpu)
+static int cpu_exists(const int cpu)
 {
 	char path[PATH_MAX];
 
@@ -104,7 +93,7 @@ static int cpu_exists(int cpu)
 }
 #endif
 
-static void set_HZ(fwts_framework *fw, const int cpu, const unsigned long Hz)
+static void set_HZ(fwts_framework *fw, const int cpu, const uint32_t Hz)
 {
 	cpu_set_t mask, oldset;
 	char path[PATH_MAX];
@@ -122,11 +111,10 @@ static void set_HZ(fwts_framework *fw, const int cpu, const unsigned long Hz)
 
 	/* then set the speed */
 	cpu_mkpath(path, sizeof(path), cpu, "scaling_setspeed");
-	snprintf(buffer, sizeof(buffer), "%lu", Hz);
+	snprintf(buffer, sizeof(buffer), "%" PRIu32 , Hz);
 	fwts_set(buffer, path);
 
 	sched_setaffinity(0, sizeof(oldset), &oldset);
-
 }
 
 #ifdef FWTS_ARCH_INTEL
@@ -183,7 +171,7 @@ static int get_performance_repeat(
 }
 #endif
 
-static char *HzToHuman(unsigned long hz)
+static char *hz_to_human(const uint32_t hz)
 {
 	static char buffer[32];
 	unsigned long long Hz;
@@ -191,15 +179,15 @@ static char *HzToHuman(unsigned long hz)
 	Hz = hz;
 
 	if (Hz > 1500000) {
-		snprintf(buffer, sizeof(buffer), "%6.2f GHz",
+		snprintf(buffer, sizeof(buffer), "%.2f GHz",
 			(Hz+50000.0) / 1000000);
 		return buffer;
 	} else if (Hz > 1000) {
-		snprintf(buffer, sizeof(buffer), "%6lli MHz",
+		snprintf(buffer, sizeof(buffer), "%llu MHz",
 			(Hz+500) / 1000);
 		return buffer;
 	} else {
-		snprintf(buffer, sizeof(buffer), "%9lli", Hz);
+		snprintf(buffer, sizeof(buffer), "%llu", Hz);
 		return buffer;
 	}
 }
@@ -219,6 +207,21 @@ static uint32_t get_claimed_hz(const int cpu)
 	return value;
 }
 
+static uint32_t get_bios_limit(const int cpu)
+{
+	char path[PATH_MAX];
+	char *buffer;
+	uint32_t value = 0;
+
+	cpu_mkpath(path, sizeof(path), cpu, "bios_limit");
+
+	if ((buffer = fwts_get(path)) != NULL) {
+		value = strtoul(buffer, NULL, 10);
+		free(buffer);
+	}
+	return value;
+}
+
 static int cpu_freq_compare(const void *v1, const void *v2)
 {
 	const fwts_cpu_freq *cpu_freq1 = (fwts_cpu_freq *)v1;
@@ -227,44 +230,23 @@ static int cpu_freq_compare(const void *v1, const void *v2)
 	return cpu_freq1->Hz - cpu_freq2->Hz;
 }
 
-static void do_cpu(fwts_framework *fw, int cpu)
+static int read_freqs_available(const int cpu, fwts_cpu_freq *freqs)
 {
 	char path[PATH_MAX];
 	char line[4096];
-	fwts_cpu_freq freqs[MAX_FREQS];
 	FILE *file;
 	char *c, *c2;
-	int i;
-	int speedcount;
-	static int warned=0;
-	int warned_PSS = 0;
-	uint64_t cpu_top_speed = 0;
+	int i = 0;
 
-	memset(freqs, 0, sizeof(freqs));
 	memset(line, 0, sizeof(line));
-
-	set_governor(fw, cpu);
-
 	cpu_mkpath(path, sizeof(path), cpu, "scaling_available_frequencies");
-	if ((file = fopen(path, "r")) == NULL) {
-		if (!no_cpufreq) {
-			fwts_log_warning(fw, "Frequency scaling not supported.");
-			no_cpufreq = 1;
-		}
-		return;
-	}
-	if (fgets(line, 4095, file) == NULL) {
-		fclose(file);
-		return;
-	}
+	if ((file = fopen(path, "r")) == NULL)
+		return 0;
+	c = fgets(line, 4095, file);
 	fclose(file);
+	if (!c)
+		return 0;
 
-	if (total_tests == 1)
-		total_tests = (2+count_ints(line)) *
-			num_cpus + 2;
-
-	c = line;
-	i = 0;
 	while ((i < MAX_FREQS) && c && strlen(c) > 1) {
 		c2 = strchr(c, ' ');
 		if (c2) {
@@ -274,8 +256,56 @@ static void do_cpu(fwts_framework *fw, int cpu)
 			c2 = NULL;
 
 		freqs[i].Hz = strtoull(c, NULL, 10);
+		c = c2;
+		i++;
+	}
+	return i;
+}
+
+static void do_cpu(fwts_framework *fw, const int cpu)
+{
+	fwts_cpu_freq freqs[MAX_FREQS];
+	int i, speedcount;
+	static int warned = 0;
+	bool warned_PSS = false;
+	uint64_t cpu_top_speed = 0;
+	int claimed_hz_too_low = 0;
+	int bios_limit_too_low = 0;
+	const uint32_t claimed_hz = get_claimed_hz(cpu);
+	const uint32_t bios_limit = get_bios_limit(cpu);
+
+	memset(freqs, 0, sizeof(freqs));
+	set_governor(fw, cpu);
+
+	if ((speedcount = read_freqs_available(cpu, freqs)) == 0) {
+		if (!no_cpufreq) {
+			char path[PATH_MAX];
+			char *driver;
+
+			no_cpufreq = true;
+			fwts_warning(fw, "CPU frequency scaling not supported.");
+
+			cpu_mkpath(path, sizeof(path), cpu, "scaling_driver");
+			driver = fwts_get(path);
+			if (driver) {
+				fwts_advice(fw, "Scaling driver '%s' is enabled and this "
+					"does not seem to allow CPU frequency scaling.", driver);
+				free(driver);
+			}
+		}
+		return;
+	}
+	if (total_tests == 1)
+		total_tests = ((2 + speedcount) * num_cpus) + 2;
+
+	for (i = 0; i < speedcount; i++) {
 		set_HZ(fw, cpu, freqs[i].Hz);
 
+		if ((claimed_hz != 0) && (claimed_hz < freqs[i].Hz))
+			claimed_hz_too_low++;
+		if ((bios_limit != 0) && (bios_limit < freqs[i].Hz))
+			bios_limit_too_low++;
+
 		if (fwts_cpu_performance(fw, cpu, &freqs[i].speed) != FWTS_OK) {
 			fwts_log_error(fw, "Failed to get CPU performance for "
 				"CPU frequency %" PRIu32 " Hz.", freqs[i].Hz);
@@ -286,21 +316,41 @@ static void do_cpu(fwts_framework *fw, int cpu)
 
 		performed_tests++;
 		fwts_progress(fw, 100 * performed_tests/total_tests);
-
-		i++;
-		c = c2;
 	}
-	speedcount = i;
 
 	if (cpu_top_speed > top_speed)
 		top_speed = cpu_top_speed;
 
+	if (claimed_hz_too_low) {
+		char path[PATH_MAX];
+
+		cpu_mkpath(path, sizeof(path), cpu, "scaling_max_freq");
+		fwts_warning(fw,
+			"There were %d CPU frequencies larger than the _PSS "
+			"maximum CPU frequency of %s for CPU %d. Has %s "
+			"been set too low?",
+			claimed_hz_too_low, hz_to_human(claimed_hz), cpu, path);
+	}
+
+	if (bios_limit_too_low) {
+		char path[PATH_MAX];
+
+		cpu_mkpath(path, sizeof(path), cpu, "bios_limit");
+		fwts_warning(fw,
+			"The CPU frequency BIOS limit %s for CPU %d was set to %s "
+			"which is lower than some of the ACPI scaling frequencies.",
+			path, cpu, hz_to_human(bios_limit));
+	}
+
+	if (claimed_hz_too_low || bios_limit_too_low)
+		fwts_log_nl(fw);
+
 	fwts_log_info(fw, "CPU %d: %i CPU frequency steps supported.", cpu, speedcount);
 	fwts_log_info_verbatum(fw, " Frequency | Relative Speed | Bogo loops");
 	fwts_log_info_verbatum(fw, "-----------+----------------+-----------");
 	for (i = 0; i < speedcount; i++)
-		fwts_log_info_verbatum(fw, "%9s |     %5.1f %%    | %9" PRIu64,
-			HzToHuman(freqs[i].Hz),
+		fwts_log_info_verbatum(fw, "%10s |     %5.1f %%    | %9" PRIu64,
+			hz_to_human(freqs[i].Hz),
 			100.0 * freqs[i].speed/cpu_top_speed,
 			freqs[i].speed);
 
@@ -314,7 +364,7 @@ static void do_cpu(fwts_framework *fw, int cpu)
 			"CPUFreqPStates",
 			"Not all processors support the same number of P states.");
 
-	if (speedcount<2)
+	if (speedcount < 2)
 		return;
 
 	/* Sort the frequencies */
@@ -332,14 +382,15 @@ static void do_cpu(fwts_framework *fw, int cpu)
 				"Supposedly higher frequency %s is slower (%" PRIu64
 				" bogo loops) than frequency %s (%" PRIu64
 				" bogo loops) on CPU %i.",
-				HzToHuman(freqs[i+1].Hz), freqs[i+1].speed,
-				HzToHuman(freqs[i].Hz), freqs[i].speed,
+				hz_to_human(freqs[i+1].Hz), freqs[i+1].speed,
+				hz_to_human(freqs[i].Hz), freqs[i].speed,
 				cpu);
-		if (freqs[i].Hz > get_claimed_hz(cpu) && !warned_PSS) {
-			warned_PSS = 1;
+
+		if ((freqs[i].Hz > claimed_hz) && !warned_PSS) {
+			warned_PSS = true;
 			fwts_warning(fw, "Frequency %" PRIu32
 				" not achievable; _PSS limit of %" PRIu32 " in effect?",
-				freqs[i].Hz, get_claimed_hz(cpu));
+				freqs[i].Hz, claimed_hz);
 		}
 	}
 }
@@ -349,9 +400,8 @@ static void lowest_speed(fwts_framework *fw, const int cpu)
 {
 	char path[PATH_MAX];
 	char *line;
-	unsigned long Hz;
 	char *c, *c2;
-	unsigned long lowspeed=0;
+	uint32_t Hz, lowspeed = 0;
 
 	cpu_mkpath(path, sizeof(path), cpu, "scaling_available_frequencies");
 	if ((line = fwts_get(path)) == NULL)
@@ -705,7 +755,7 @@ static int cpufreq_test1(fwts_framework *fw)
 static int cpufreq_init(fwts_framework *fw)
 {
 	if ((num_cpus = fwts_cpu_enumerate()) == FWTS_ERROR) {
-		fwts_log_warning(fw, "Cannot determine number of CPUS, defaulting to 1.");
+		fwts_warning(fw, "Cannot determine number of CPUS, defaulting to 1.");
 		num_cpus = 1;
 	}
 	return FWTS_OK;
@@ -713,9 +763,9 @@ static int cpufreq_init(fwts_framework *fw)
 
 static fwts_framework_minor_test cpufreq_tests[] = {
 #ifdef FWTS_ARCH_INTEL
-	{ cpufreq_test1, "CPU P-State Checks." },
+	{ cpufreq_test1, "CPU P-State tests." },
 #else
-	{ cpufreq_test1, "CPU Frequency Checks." },
+	{ cpufreq_test1, "CPU Frequency tests." },
 #endif
 	{ NULL, NULL }
 };
