@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006, Intel Corporation
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This file was derived from part of the Linux-ready Firmware Developer Kit,
  * however, it has been completely re-written.
@@ -288,7 +288,7 @@ static int microcode_test1(fwts_framework *fw)
 		if (passed > 0)
 			fwts_passed(fw, "%d CPU(s) have the latest microcode loaded.", passed);
 		else
-			fwts_log_info(fw, "Could not determine from kernel log if latest microcode has been loaded.");
+			fwts_skipped(fw, "Could not determine from kernel log if latest microcode has been loaded.");
 	}
 
 	closedir(dir);
@@ -299,12 +299,12 @@ static int microcode_test1(fwts_framework *fw)
 }
 
 static fwts_framework_minor_test microcode_tests[] = {
-	{ microcode_test1, "Check for most recent microcode being loaded." },
+	{ microcode_test1, "Test for most recent microcode being loaded." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops microcode_ops = {
-	.description = "Check if system is using latest microcode.",
+	.description = "Test if system is using latest microcode.",
 	.init        = microcode_init,
 	.deinit      = microcode_deinit,
 	.minor_tests = microcode_tests
