@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2006, Intel Corp
  * Copyright (C) 2007, AMD Inc
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This code was originally part of the Linux-ready Firmware Developer Kit
  *
@@ -78,12 +78,12 @@ static int virt_test1(fwts_framework *fw)
 
 
 static fwts_framework_minor_test virt_tests[] = {
-	{ virt_test1, "Check CPU Virtualisation Configuration." },
+	{ virt_test1, "CPU Virtualisation Configuration test." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops virt_ops = {
-	.description = "Test CPU Virtualisation Configuration.",
+	.description = "CPU Virtualisation Configuration test.",
 	.init        = virt_init,
 	.deinit      = virt_deinit,
 	.minor_tests = virt_tests
