@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006, Intel Corporation
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This code was originally part of the Linux-ready Firmware Developer Kit
  *
@@ -180,7 +180,7 @@ static int hpet_check_test1(fwts_framework *fw)
 	}
 
 	if (hpet_base_p == 0) {
-		fwts_log_info(fw, "No base address found for HPET.");
+		fwts_log_info(fw, "No base address found for HPET in the kernel log.");
 		return FWTS_ERROR;
 	}
 
@@ -357,11 +357,16 @@ static int hpet_check_test3(fwts_framework *fw)
 {
 	int i;
 
+	if (hpet_base_p == 0) {
+		fwts_log_info(fw, "Test skipped because HPET address was not found.");
+		return FWTS_SKIP;
+	}
+
 	hpet_check_base_acpi_table(fw, "DSDT", 0);
 
-	for (i=0;i<11;i++) {
+	for (i = 0; i< 11; i++)
 		hpet_check_base_acpi_table(fw, "SSDT", i);
-	}
+
 	return FWTS_OK;
 }
 
@@ -373,7 +378,7 @@ static int hpet_check_test4(fwts_framework *fw)
 	uint32_t clk_period;
 
 	if (hpet_base_p == 0) {
-		fwts_log_info(fw, "Test skipped because previous test failed.");
+		fwts_log_info(fw, "Test skipped because HPET address was not found.");
 		return FWTS_SKIP;
 	}
 
@@ -408,15 +413,15 @@ static int hpet_check_test4(fwts_framework *fw)
 
 
 static fwts_framework_minor_test hpet_check_tests[] = {
-	{ hpet_check_test1, "Check HPET base in kernel log." },
-	{ hpet_check_test2, "Check HPET base in HPET table. "},
-	{ hpet_check_test3, "Check HPET base in DSDT and/or SSDT. "},
-	{ hpet_check_test4, "Sanity check HPET configuration." },
+	{ hpet_check_test1, "Test HPET base in kernel log." },
+	{ hpet_check_test2, "Test HPET base in HPET table. "},
+	{ hpet_check_test3, "Test HPET base in DSDT and/or SSDT. "},
+	{ hpet_check_test4, "Test HPET configuration." },
 	{ NULL, NULL }
 };
 
 static fwts_framework_ops hpet_check_ops = {
-	.description = "HPET configuration test.",
+	.description = "HPET configuration tests.",
 	.init        = hpet_check_init,
 	.deinit      = hpet_check_deinit,
 	.minor_tests = hpet_check_tests
