@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,7 +30,6 @@
 
 #include "fwts_version.h"
 #include "fwts_types.h"
-#include "fwts_tag.h"
 #include "fwts_binpaths.h"
 #include "fwts_framework.h"
 #include "fwts_log.h"
@@ -55,7 +54,6 @@
 #include "fwts_stringextras.h"
 #include "fwts_tty.h"
 #include "fwts_wakealarm.h"
-#include "fwts_virt.h"
 #include "fwts_formatting.h"
 #include "fwts_summary.h"
 #include "fwts_mmap.h"
@@ -79,5 +77,6 @@
 #include "fwts_json.h"
 #include "fwts_ioport.h"
 #include "fwts_release.h"
+#include "fwts_pci.h"
 
 #endif
