@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -42,7 +42,7 @@
 #include "fwts_framework.h"
 #include "fwts_log.h"
 
-extern char *fwts_acpi_fadt_preferred_pm_profile[];
+extern const char *fwts_acpi_fadt_preferred_pm_profile[];
 
 #define FWTS_ACPI_FADT_PREFERRED_PM_PROFILE(x)		\
 	((x) > 8) ? "Reserved" : fwts_acpi_fadt_preferred_pm_profile[x]
@@ -681,6 +681,77 @@ typedef struct {
 	uint16_t	min_request_turnaround_time;
 } __attribute__ ((packed)) fwts_acpi_table_pcct_subspace_type_0;
 
+typedef struct {
+	fwts_acpi_table_header	header;
+	uint8_t		interface_type;
+	uint8_t		reserved1[3];
+	fwts_acpi_gas	base_address;
+	uint8_t		interrupt_type;
+	uint8_t		irq;
+	uint32_t	gsi;
+	uint8_t		baud_rate;
+	uint8_t		parity;
+	uint8_t		stop_bits;
+	uint8_t		flow_control;
+	uint8_t		terminal_type;
+	uint8_t		reserved2;
+	uint16_t	pci_device_id;
+	uint16_t	pci_vendor_id;
+	uint8_t		pci_bus_number;
+	uint8_t		pci_device_number;
+	uint8_t		pci_function_number;
+	uint32_t	pci_flags;
+	uint8_t		pci_segment;
+	uint32_t	reserved3;
+} __attribute__ ((packed)) fwts_acpi_table_spcr;
+
+typedef struct {
+	fwts_acpi_table_header	header;
+	uint8_t		interface_type;
+	uint8_t		reserved1[3];
+	fwts_acpi_gas	base_address;
+} __attribute__ ((packed)) fwts_acpi_table_dbgp;
+
+typedef struct {
+	fwts_acpi_table_header	header;
+	uint32_t	info_offset;
+	uint32_t	info_count;
+} __attribute__ ((packed)) fwts_acpi_table_dbg2;
+
+typedef struct {
+	uint8_t		revision;
+	uint16_t	length;
+	uint8_t		number_of_regs;
+	uint16_t	namespace_length;
+	uint16_t	namespace_offset;
+	uint16_t	oem_data_length;
+	uint16_t	oem_data_offset;
+	uint16_t	port_type;
+	uint16_t	port_subtype;
+	uint16_t	reserved;
+	uint16_t	base_address_offset;
+	uint16_t	address_size_offset;
+} __attribute__ ((packed)) fwts_acpi_table_dbg2_info;
+
+/*
+ *  http://www.dmtf.org/standards/published_documents/DSP0256_1.0.0.pdf
+ */
+typedef struct {
+	fwts_acpi_table_header	header;
+	uint8_t		interface_type;
+	uint8_t		protocol_identifier;
+	uint64_t	protocol_data;
+	uint8_t		interrupt_type;
+	uint8_t		gpe;
+	uint8_t		pci_device_flag;
+	uint32_t	global_system_interrupt;
+	fwts_acpi_gas	base_address;
+	uint8_t		pci_segment_group_number;
+	uint8_t		pci_bus_number;
+	uint8_t		pci_device_number;
+	uint8_t		pci_function_number;
+} __attribute__ ((packed)) fwts_acpi_table_mchi;
+
 void fwts_acpi_table_get_header(fwts_acpi_table_header *hdr, uint8_t *data);
 
 #endif
