@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -50,9 +50,7 @@ typedef enum {
 	FWTS_FLAG_TEST_ACPI			= 0x04000000,
 	FWTS_FLAG_UTILS				= 0x08000000,
 	FWTS_FLAG_QUIET				= 0x10000000,
-	FWTS_FLAG_LP_TAGS			= 0x20000000,
-	FWTS_FLAG_LP_TAGS_LOG			= 0x40000000,
-	FWTS_FLAG_SHOW_TESTS_FULL		= 0x80000000,
+	FWTS_FLAG_SHOW_TESTS_FULL		= 0x20000000,
 } fwts_framework_flags;
 
 #define FWTS_FLAG_TEST_MASK		\
@@ -132,9 +130,6 @@ typedef struct {
 	bool print_summary;			/* Print summary of results at end of test runs */
 	fwts_log_level failed_level;		/* Bit mask of failed levels in test run */
 
-	fwts_list test_taglist;			/* List of tags found when running all minor tests */
-	fwts_list total_taglist;		/* List of tags found when running all tests */
-
 	int firmware_type;			/* Type of firmware */
 	bool show_progress;			/* Show progress while running current test */
 
@@ -144,6 +139,7 @@ typedef struct {
 	fwts_list errors_filter_discard;	/* Results to discard, empty = discard none */
 	bool error_filtered_out;		/* True if a klog message has been filtered out */
 	fwts_acpica_mode acpica_mode;		/* ACPICA mode flags */
+	void *rsdp;				/* ACPI RSDP address */
 } fwts_framework;
 
 typedef struct {
