@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <stdint.h>
 
-void fwts_guid_buf_to_str(uint8_t *guid, char *guid_str, const size_t guid_str_len);
+void fwts_guid_buf_to_str(const uint8_t *guid, char *guid_str, const size_t guid_str_len);
 void fwts_guid_str_to_buf(const char *guid_str, uint8_t *guid, const size_t guid_len);
 
 #endif
