@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2013 Canonical
+ * Copyright (C) 2011-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,14 +23,11 @@
 #include "fwts.h"
 
 typedef struct {
-	fwts_list *network;
-	fwts_list *ethernet;
-	fwts_list *ifconfig;
-	fwts_list *iwconfig;
-	fwts_list *hciconfig;
-	fwts_list *videocard;
-	fwts_list *xinput;
-	fwts_list *pactl;
+	fwts_list network;	/* PCI network config */
+	fwts_list videocard;	/* PCI video card config */
+	fwts_list netdevs;	/* Network devices */
+	fwts_list input;	/* Input device config */
+	fwts_list bluetooth;	/* Bluetooth config */
 } fwts_hwinfo;
 
 int fwts_hwinfo_get(fwts_framework *fw, fwts_hwinfo *hwinfo);
