@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,9 +22,10 @@
 
 #include "fwts.h"
 
-int fwts_iasl_disassemble_all_to_file(fwts_framework *fw);
+int fwts_iasl_disassemble_all_to_file(fwts_framework *fw,
+	const char *path);
 
-int fwts_iasl_disassemble(fwts_framework *fw, 
+int fwts_iasl_disassemble(fwts_framework *fw,
 	const char *table,
 	const int which,
 	fwts_list **ias_output);
@@ -33,6 +34,7 @@ int fwts_iasl_reassemble(fwts_framework *fw,
 	const uint8_t *data,
 	const int len,
 	fwts_list **iasl_disassembly,
-	fwts_list **iasl_errors);
+	fwts_list **iasl_stdout,
+	fwts_list **iasl_stderr);
 
 #endif
