@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,7 +20,8 @@
 #ifndef __FWTS_UEFI_H__
 #define __FWTS_UEFI_H__
 
-#define FWTS_UEFI_LOAD_ACTIVE 0x00000001
+#define FWTS_UEFI_LOAD_OPTION_ACTIVE 		0x00000001
+#define FWTS_UEFI_LOAD_OPTION_FORCE_RECONNECT 	0x00000002
 
 typedef struct {
 	uint16_t	*varname;
@@ -94,6 +95,60 @@ enum {
 #define EFI_OS_INDICATIONS_FMP_CAPSULE_SUPPORTED 		0x0000000000000008
 #define EFI_OS_INDICATIONS_CAPSULE_RESULT_VAR_SUPPORTED		0x0000000000000010
 
+#define EFI_CERT_SHA256_GUID \
+{ 0xc1c41626, 0x504c, 0x4092, { 0xac, 0xa9, 0x41, 0xf9, 0x36, 0x93, 0x43, 0x28 }}
+
+#define EFI_CERT_RSA2048_GUID \
+{ 0x3c5766e8, 0x269c, 0x4e34, { 0xaa, 0x14, 0xed, 0x77, 0x6e, 0x85, 0xb3, 0xb6 }}
+
+#define EFI_CERT_RSA2048_SHA256_GUID \
+{ 0xe2b36190, 0x879b, 0x4a3d, { 0xad, 0x8d, 0xf2, 0xe7, 0xbb, 0xa3, 0x27, 0x84 }}
+
+#define EFI_CERT_SHA1_GUID \
+{ 0x826ca512, 0xcf10, 0x4ac9, { 0xb1, 0x87, 0xbe, 0x1, 0x49, 0x66, 0x31, 0xbd }}
+
+#define EFI_CERT_RSA2048_SHA1_GUID \
+{ 0x67f8444f, 0x8743, 0x48f1, { 0xa3, 0x28, 0x1e, 0xaa, 0xb8, 0x73, 0x60, 0x80 }}
+
+#define EFI_CERT_X509_GUID \
+{ 0xa5c059a1, 0x94e4, 0x4aa7, { 0x87, 0xb5, 0xab, 0x15, 0x5c, 0x2b, 0xf0, 0x72 }}
+
+#define EFI_CERT_SHA224_GUID \
+{ 0xb6e5233, 0xa65c, 0x44c9, { 0x94, 0x7, 0xd9, 0xab, 0x83, 0xbf, 0xc8, 0xbd }}
+
+#define EFI_CERT_SHA384_GUID \
+{ 0xff3e5307, 0x9fd0, 0x48c9, { 0x85, 0xf1, 0x8a, 0xd5, 0x6c, 0x70, 0x1e, 0x1 }}
+
+#define EFI_CERT_SHA512_GUID \
+{ 0x93e0fae, 0xa6c4, 0x4f50, { 0x9f, 0x1b, 0xd4, 0x1e, 0x2b, 0x89, 0xc1, 0x9a }}
+
+#define EFI_CERT_X509_SHA256_GUID \
+{ 0x3bd2a492, 0x96c0, 0x4079, { 0xb4, 0x20, 0xfc, 0xf9, 0x8e, 0xf1, 0x03, 0xed }}
+
+#define EFI_CERT_X509_SHA384_GUID \
+{ 0x7076876e, 0x80c2, 0x4ee6, { 0xaa, 0xd2, 0x28, 0xb3, 0x49, 0xa6, 0x86, 0x5b }}
+
+#define EFI_CERT_X509_SHA512_GUID \
+{ 0x446dbf63, 0x2502, 0x4cda, { 0xbc, 0xfa, 0x24, 0x65, 0xd2, 0xb0, 0xfe, 0x9d }}
+
+#define EFI_PC_ANSI_GUID \
+{ 0xe0c14753, 0xf9be, 0x11d2, { 0x9a, 0x0c, 0x00, 0x90, 0x27, 0x3f, 0xc1, 0x4d }}
+
+#define EFI_VT_100_GUID \
+{ 0xdfa66065, 0xb419, 0x11d3, { 0x9a, 0x2d, 0x00, 0x90, 0x27, 0x3f, 0xc1, 0x4d }}
+
+#define EFI_VT_100_PLUS_GUID \
+{ 0x7baec70b, 0x57e0, 0x4c76, { 0x8e, 0x87, 0x2f, 0x9e, 0x28, 0x08, 0x83, 0x43 }}
+
+#define EFI_VT_UTF8_GUID \
+{ 0xad15a0d6, 0x8bec, 0x4acf, { 0xa0, 0x73, 0xd0, 0x1d, 0xe7, 0x7e, 0x2d, 0x88 }}
+
+#define EFI_UART_DEVICE_PATH_GUID \
+{ 0x37499a9d, 0x542f, 0x4c89, { 0xa0, 0x26, 0x35, 0xda, 0x14, 0x20, 0x94, 0xe4 }}
+
+#define EFI_SAS_DEVICE_PATH_GUID \
+{ 0xd487ddb4, 0x008b, 0x11d9, { 0xaf, 0xdc, 0x00, 0x10, 0x83, 0xff, 0xca, 0x4d }}
+
 #if 0
 typedef struct {
 	char *description;
@@ -114,6 +169,17 @@ typedef struct {
         fwts_uefi_dev_path unused_file_path_list[1];
 } __attribute__((packed)) fwts_uefi_load_option;
 
+typedef struct {
+        uint32_t keydata;
+        uint32_t bootoptioncrc;
+        uint16_t bootoption;
+} __attribute__((packed)) fwts_uefi_key_option;
+
+typedef struct {
+        uint16_t scancode;
+        uint16_t unicodechar;
+} __attribute__((packed)) fwts_uefi_input_key;
+
 typedef enum {
 	FWTS_UEFI_HARDWARE_DEV_PATH_TYPE =		(0x01),
 	FWTS_UEFI_ACPI_DEVICE_PATH_TYPE =		(0x02),
@@ -138,7 +204,8 @@ typedef enum {
 
 typedef enum {
 	FWTS_UEFI_ACPI_DEVICE_PATH_SUBTYPE = 		(0x01),
-	FWTS_UEFI_EXPANDED_ACPI_DEVICE_PATH_SUBTYPE = 	(0x02)
+	FWTS_UEFI_EXPANDED_ACPI_DEVICE_PATH_SUBTYPE = 	(0x02),
+	FWTS_UEFI_ACPI_ADR_DEVICE_PATH_SUBTYPE = 	(0x03)
 } acpi_dev_path_subtypes;
 
 typedef enum {
@@ -154,7 +221,15 @@ typedef enum {
 	FWTS_UEFI_IPV4_DEVICE_PATH_SUBTYPE =		(0x0c),
 	FWTS_UEFI_IPV6_DEVICE_PATH_SUBTYPE =		(0x0d),
 	FWTS_UEFI_UART_DEVICE_PATH_SUBTYPE =		(0x0e),
-	FWTS_UEFI_USB_CLASS_DEVICE_PATH_SUBTYPE =	(0x0f)
+	FWTS_UEFI_USB_CLASS_DEVICE_PATH_SUBTYPE =	(0x0f),
+	FWTS_UEFI_USB_WWID_DEVICE_PATH_SUBTYPE =	(0x10),
+	FWTS_UEFI_LOGICAL_UNIT_DEVICE_PATH_SUBTYPE =	(0x11),
+	FWTS_UEFI_SATA_DEVICE_PATH_SUBTYPE = 		(0x12),
+	FWTS_UEFI_ISCSI_DEVICE_PATH_SUBTYPE = 		(0x13),
+	FWTS_UEFI_VLAN_DEVICE_PATH_SUBTYPE = 		(0x14),
+	FWTS_UEFI_FIBRE_CHANNEL_EX_DEVICE_PATH_SUBTYPE = (0x15),
+	FWTS_UEFI_SAS_EX_DEVICE_PATH_SUBTYPE =		(0x16),
+	FWTS_UEFI_NVM_EXPRESS_NAMESP_DEVICE_PATH_SUBTYPE = (0x17)
 } messaging_dev_path_subtypes;
 
 typedef enum {
@@ -162,7 +237,9 @@ typedef enum {
 	FWTS_UEFI_CDROM_DEVICE_PATH_SUBTYPE =		(0x02),
 	FWTS_UEFI_VENDOR_MEDIA_DEVICE_PATH_SUBTYPE =	(0x03),
 	FWTS_UEFI_FILE_PATH_DEVICE_PATH_SUBTYPE =	(0x04),
-	FWTS_UEFI_PROTOCOL_DEVICE_PATH_SUBTYPE =	(0x05)
+	FWTS_UEFI_PROTOCOL_DEVICE_PATH_SUBTYPE =	(0x05),
+	FWTS_UEFI_PIWG_FW_FILE_DEVICE_PATH_SUBTYPE =	(0x06),
+	FWTS_UEFI_PIWG_FW_VOLUME_DEVICE_PATH_SUBTYPE =	(0x07)
 } media_dev_path_subtypes;
 
 typedef enum {
@@ -209,7 +286,7 @@ typedef struct {
 	fwts_uefi_dev_path dev_path;
 	uint32_t hid;
 	uint32_t uid;
-} fwts_uefi_acpi_dev_path;
+} __attribute__((packed)) fwts_uefi_acpi_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
@@ -217,20 +294,25 @@ typedef struct {
 	uint32_t uid;
 	uint32_t cid;
 	char hidstr[1];
-} fwts_uefi_expanded_acpi_dev_path;
+} __attribute__((packed)) fwts_uefi_expanded_acpi_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	uint32_t adr;
+} __attribute__((packed)) fwts_uefi_acpi_adr_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
 	uint8_t primary_secondary;
 	uint8_t slave_master;
 	uint16_t lun;
-} fwts_uefi_atapi_dev_path;
+} __attribute__((packed)) fwts_uefi_atapi_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
 	uint16_t pun;
 	uint16_t lun;
-} fwts_uefi_scsi_dev_path;
+} __attribute__((packed)) fwts_uefi_scsi_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
@@ -243,13 +325,13 @@ typedef struct {
 	fwts_uefi_dev_path dev_path;
 	uint32_t reserved;
 	uint64_t guid;
-} fwts_uefi_1394_dev_path;
+} __attribute__((packed)) fwts_uefi_1394_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
 	uint8_t parent_port_number;
 	uint8_t interface;
-} fwts_uefi_usb_dev_path;
+} __attribute__((packed)) fwts_uefi_usb_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
@@ -258,19 +340,19 @@ typedef struct {
 	uint8_t device_class;
 	uint8_t device_subclass;
 	uint8_t device_protocol;
-} fwts_uefi_usb_class_dev_path;
+} __attribute__((packed)) fwts_uefi_usb_class_dev_path;
 
 typedef struct
 {
 	fwts_uefi_dev_path dev_path;
 	uint32_t tid;
-} fwts_uefi_i2o_dev_path;
+} __attribute__((packed)) fwts_uefi_i2o_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
   	fwts_uefi_mac_addr mac_addr;
 	uint8_t if_type;
-} fwts_uefi_mac_addr_dev_path;
+} __attribute__((packed)) fwts_uefi_mac_addr_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
@@ -280,7 +362,7 @@ typedef struct {
   	uint16_t remote_port;
   	uint16_t protocol;
   	uint8_t static_ip_address;
-} fwts_uefi_ipv4_dev_path;
+} __attribute__((packed)) fwts_uefi_ipv4_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
@@ -290,7 +372,7 @@ typedef struct {
   	uint16_t remote_port;
   	uint16_t protocol;
   	uint8_t static_ip_address;
-} fwts_uefi_ipv6_dev_path;
+} __attribute__((packed)) fwts_uefi_ipv6_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
@@ -299,7 +381,7 @@ typedef struct {
 	uint64_t remote_id;
 	uint64_t target_port_id;
 	uint64_t device_id;
-} fwts_uefi_infiniband_dev_path;
+} __attribute__((packed)) fwts_uefi_infiniband_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
@@ -308,13 +390,78 @@ typedef struct {
 	uint8_t data_bits;
 	uint8_t parity;
 	uint8_t stop_bits;
-} fwts_uefi_uart_dev_path;
+} __attribute__((packed)) fwts_uefi_uart_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
 	fwts_uefi_guid guid;
 	uint8_t vendor_defined_data[0];
-} fwts_uefi_vendor_messaging_dev_path;
+} __attribute__((packed)) fwts_uefi_vendor_messaging_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	fwts_uefi_guid guid;
+	uint32_t reserved;
+	uint64_t sas_addr;
+	uint64_t lun;
+	uint16_t dev_topology_info;
+	uint16_t rtp;
+} __attribute__((packed)) fwts_uefi_sas_messaging_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	uint32_t reserved;
+	uint64_t wwn;
+	uint64_t lun;
+} __attribute__((packed)) fwts_uefi_fibre_channel_ex_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	uint16_t hbapn;
+	uint16_t pmpn;
+	uint16_t lun;
+} __attribute__((packed)) fwts_uefi_sata_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	uint16_t interface_num;
+	uint16_t vendor_id;
+	uint16_t product_id;
+	uint16_t serial_number[0];
+} __attribute__((packed)) fwts_uefi_usb_wwid_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	uint8_t lun;
+} __attribute__((packed)) fwts_uefi_logical_unit_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	uint16_t vlanid;
+} __attribute__((packed)) fwts_uefi_vlan_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	uint64_t sas_addr;
+	uint64_t lun;
+	uint16_t dev_topology_info;
+	uint16_t rtp;
+} __attribute__((packed)) fwts_uefi_sas_ex_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	uint16_t protocol;
+	uint16_t options;
+	uint64_t lun;
+	uint16_t tpg_tag;
+	char iscsi_tn[0];
+} __attribute__((packed)) fwts_uefi_iscsi_dev_path;
+ 
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	uint32_t namesp_id;
+	uint64_t ext_unique_id;
+} __attribute__((packed)) fwts_uefi_nvm_express_namespace_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
@@ -324,32 +471,54 @@ typedef struct {
 	uint8_t partition_signature[8];
 	uint8_t mbr_type;
 	uint8_t signature_type;
-} fwts_uefi_hard_drive_dev_path;
+} __attribute__((packed)) fwts_uefi_hard_drive_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
 	uint32_t boot_entry;
 	uint64_t partition_start;
 	uint64_t partition_size;
-} fwts_uefi_cdrom_dev_path;
+} __attribute__((packed)) fwts_uefi_cdrom_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
 	fwts_uefi_guid guid;
 	uint8_t vendor_defined_data[0];
-} fwts_uefi_vendor_media_dev_path;
+} __attribute__((packed)) fwts_uefi_vendor_media_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
 	uint16_t path_name[0];
-} fwts_uefi_file_path_dev_path;
+} __attribute__((packed)) fwts_uefi_file_path_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	fwts_uefi_guid protocol_guid;
+} __attribute__((packed)) fwts_media_protocol_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	fwts_uefi_guid fw_file_name;
+} __attribute__((packed)) fwts_piwg_fw_file_dev_path;
+
+typedef struct {
+	fwts_uefi_dev_path dev_path;
+	fwts_uefi_guid fw_volume_name;
+} __attribute__((packed)) fwts_piwg_fw_volume_dev_path;
 
 typedef struct {
 	fwts_uefi_dev_path dev_path;
 	uint16_t device_type;
 	uint16_t status_flags;
   	char description[0];
-} fwts_uefi_bios_dev_path;
+} __attribute__((packed)) fwts_uefi_bios_dev_path;
+
+typedef struct {
+	fwts_uefi_guid signaturetype;
+	uint32_t signaturelistsize;
+	uint32_t signatureheadersize;
+	uint32_t signaturesize;
+} __attribute__((packed)) fwts_uefi_signature_list;
 
 void fwts_uefi_str16_to_str(char *dst, const size_t len, const uint16_t *src);
 size_t fwts_uefi_str16len(const uint16_t *str);
