@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,6 +22,8 @@
 #include <stddef.h>
 #include <stdbool.h>
 #include <string.h>
+#include <stdint.h>
+#include <inttypes.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -40,17 +42,52 @@
 #define ACPI_MAX_TABLES	(64)			/* Max number of ACPI tables */
 
 static fwts_acpi_table_info	tables[ACPI_MAX_TABLES];
-static int acpi_tables_loaded = 0;
+
+typedef enum {
+	ACPI_TABLES_NOT_LOADED		= 0,
+	ACPI_TABLES_LOADED_OK		= 1,
+	ACPI_TABLES_LOADED_FAILED	= 2
+} acpi_table_load_state;
+
+static acpi_table_load_state acpi_tables_loaded = ACPI_TABLES_NOT_LOADED;
 
 /*
  *  fwts_acpi_find_rsdp_efi()
  *  	Get RSDP address from EFI if possible
  */
-static void *fwts_acpi_find_rsdp_efi(void)
+static inline void *fwts_acpi_find_rsdp_efi(void)
 {
 	return fwts_scan_efi_systab("ACPI20");
 }
 
+/*
+ *  fwts_acpi_find_rsdp_klog()
+ *	Get RSDP by parsing kernel log
+ */
+static void *fwts_acpi_find_rsdp_klog(void)
+{
+	fwts_list *klog;
+	fwts_list_link *item;
+	void *rsdp = NULL;
+
+	if ((klog = fwts_klog_read()) == NULL)
+		return NULL;
+
+	fwts_list_foreach(item, klog) {
+		char *text = fwts_text_list_text(item);
+		char *ptr = strstr(text, "ACPI: RSDP");
+
+		if (ptr) {
+			rsdp = (void *)strtoul(ptr + 11, NULL, 16);
+			break;
+		}
+	}
+
+	fwts_text_list_free(klog);
+
+	return rsdp;
+}
+
 /*
  *  fwts_acpi_find_rsdp_bios()
  *	Find RSDP address by scanning BIOS memory
@@ -94,21 +131,26 @@ static void *fwts_acpi_find_rsdp_bios(void)
 static fwts_acpi_table_rsdp *fwts_acpi_get_rsdp(void *addr, size_t *rsdp_len)
 {
 	uint8_t *mem;
-	fwts_acpi_table_rsdp *rsdp;
+	fwts_acpi_table_rsdp *rsdp = NULL;
 	*rsdp_len = 0;
 
 	if ((mem = fwts_mmap((off_t)addr, sizeof(fwts_acpi_table_rsdp))) == FWTS_MAP_FAILED)
 		return NULL;
 
-	/* Determine original RSDP size from revision. */
 	rsdp = (fwts_acpi_table_rsdp*)mem;
+	/* Determine original RSDP size from revision. */
 	*rsdp_len = (rsdp->revision < 1) ? 20 : 36;
 
+	/* Must have the correct signature */
+	if (strncmp(rsdp->signature, "RSD PTR ", 8))
+		goto out;
+
 	/* Assume version 2.0 size, we don't care about a few bytes over allocation if it's version 1.0 */
 	if ((rsdp = (fwts_acpi_table_rsdp*)fwts_low_calloc(1, sizeof(fwts_acpi_table_rsdp))) == NULL)
-		return NULL;
+		goto out;
 
 	memcpy(rsdp, mem, *rsdp_len);
+out:
 	(void)fwts_munmap(mem, sizeof(fwts_acpi_table_rsdp));
 
 	return rsdp;
@@ -203,82 +245,149 @@ int fwts_acpi_free_tables(void)
 
 /*
  *  fwts_acpi_handle_fadt_tables()
- *	depending on whether 32 or 64 bit address is usable, get the FADT table
- *	address and load the FADT.
+ *	depending on whether 32 or 64 bit address is usable, get the table
+ *	address and load it. This handles the DSDT and FACS as pointed to
+ *	by the FADT
+ *
+ *	Note, we pass in the addresses of the 32 and 64 bit pointers in the
+ *	FADT because the FADT may be smaller than expected and we only want
+ *	to accesses these fields if the FADT is large enough.
  */
-static void fwts_acpi_handle_fadt_tables(
-	fwts_acpi_table_fadt *fadt,
-	const uint32_t *addr32,
-	const uint64_t *addr64,
+static int fwts_acpi_handle_fadt_tables(
+	fwts_framework *fw,
+	const fwts_acpi_table_fadt *fadt,/* FADT */
+	const char *name,		/* Name of Table addr32/addr 64 point to */
+	const char *name_addr32,	/* Name of 32 bit addr */
+	const char *name_addr64,	/* Name of 64 bit addr */
+	const uint32_t *addr32,		/* 32 bit addr */
+	const uint64_t *addr64,		/* 64 bit addr */
 	const fwts_acpi_table_provenance provenance)
 {
-	off_t addr;
+	off_t addr = 0;
 	fwts_acpi_table_header *header;
 
-	if ((addr64 != 0) && (fadt->header.length >= 140))
-		addr = (off_t)*addr64;
-	else if ((addr32 !=0) && (fadt->header.length >= 44))
+	/* newer version can have address in 64 and 32 bit pointers */
+	if ((addr64 != NULL) && (fadt->header.length >= 140)) {
+		if (*addr64 == 0) {
+			/* Work around buggy firmware, use 32 bit addr instead */
+			addr = (off_t)*addr32;
+			fwts_log_warning(fw, "FADT %s 64 bit pointer was zero, "
+				"falling back to using %s 32 bit pointer.",
+				name_addr64, name_addr32);
+		} else {
+			/* Use default 64 bit addr */
+			addr = (off_t)*addr64;
+		}
+		/* Is it sane? */
+		if (addr == 0) {
+			fwts_log_error(fw, "Failed to load %s: Cannot determine "
+				"address of %s from FADT, fields %s and %s are zero.",
+				name, name, name_addr32, name_addr64);
+			return FWTS_ERROR;
+		}
+	} else if ((addr32 != NULL) && (fadt->header.length >= 44)) {
 		addr = (off_t)*addr32;
-	else addr = 0;
-
-	if (addr) {
-		if ((header = fwts_acpi_load_table(addr)) != NULL)
-			fwts_acpi_add_table(header->signature, header,
-				(uint64_t)addr, header->length, provenance);
+		/* Is it sane? */
+		if (addr == 0)  {
+			fwts_log_error(fw, "Failed to load %s: Cannot determine "
+				"address of %s from FADT, field %s is zero.",
+				name, name, name_addr32);
+			return FWTS_ERROR;
+		}
+	} else if (fadt->header.length < 44) {
+		fwts_log_error(fw, "Failed to load %s: FADT is too small and "
+			"does not have any %s or %s fields.",
+			name, name_addr32, name_addr64);
+		return FWTS_ERROR;
+	} else {
+		/* This should not happen, addr64 or addr32 are NULL */
+		fwts_log_error(fw, "Failed to load %s: fwts error with FADT.", name);
+		return FWTS_ERROR;
 	}
+
+	/* Sane address found, load and add the table */
+	if ((header = fwts_acpi_load_table(addr)) == NULL) {
+		fwts_log_error(fw, "Could not load %s from address 0x%" PRIx64 ".",
+			name, (uint64_t)addr);
+		return FWTS_ERROR;
+	}
+	fwts_acpi_add_table(header->signature, header,
+		(uint64_t)addr, header->length, provenance);
+	return FWTS_OK;
 }
 
-static void fwts_acpi_handle_fadt(fwts_acpi_table_fadt *fadt, fwts_acpi_table_provenance provenance)
+/*
+ *  fwts_acpi_handle_fadt()
+ *	The FADT points to the FACS and DSDT with either 32 or 64 bit pointers.
+ *	Locate the FACS and DSDT tables and load them.
+ */
+static int fwts_acpi_handle_fadt(
+	fwts_framework *fw,
+	const uint64_t phys_addr,
+	const fwts_acpi_table_fadt *fadt,
+	const fwts_acpi_table_provenance provenance)
 {
-	fwts_acpi_handle_fadt_tables(fadt, &fadt->dsdt, &fadt->x_dsdt, provenance);
-	fwts_acpi_handle_fadt_tables(fadt, &fadt->firmware_control, &fadt->x_firmware_ctrl, provenance);
+	static uint64_t	facs_last_phys_addr;	/* default to zero */
+
+	/*
+	 *  The FADT handling may occur twice if it appears
+	 *  in the RSDT and the XDST, so as an optimisation
+	 *  we just need to handle it once.
+	 */
+	if (facs_last_phys_addr == phys_addr)
+		return FWTS_OK;
+
+	facs_last_phys_addr = phys_addr;
+
+	/* Determine FACS addr and load it */
+	if (fwts_acpi_handle_fadt_tables(fw, fadt,
+	    "FACS", "FIRMWARE_CTRL", "X_FIRMWARE_CTRL",
+	     &fadt->firmware_control, &fadt->x_firmware_ctrl,
+	     provenance) != FWTS_OK) {
+		return FWTS_ERROR;
+	}
+	/* Determine DSDT addr and load it */
+	if (fwts_acpi_handle_fadt_tables(fw, fadt,
+	    "DSDT", "DSTD", "X_DSDT",
+	    &fadt->dsdt, &fadt->x_dsdt, provenance) != FWTS_OK) {
+		return FWTS_ERROR;
+	}
+	return FWTS_OK;
 }
 
 /*
  *  fwts_acpi_load_tables_from_firmware()
  *  	Load up cached copies of all the ACPI tables
  */
-static int fwts_acpi_load_tables_from_firmware(void)
+static int fwts_acpi_load_tables_from_firmware(fwts_framework *fw)
 {
 	fwts_acpi_table_rsdp *rsdp;
 	fwts_acpi_table_xsdt *xsdt;
 	fwts_acpi_table_rsdt *rsdt;
 	fwts_acpi_table_header *header;
 
-	void *	rsdp_addr;
+	void *	rsdp_addr = NULL;
 	size_t	rsdp_len;
 	int 	num_entries;
 	int 	i;
 
-	/* Check for RSDP in EFI, then BIOS, if not found, give up */
-	if ((rsdp_addr = fwts_acpi_find_rsdp_efi()) == NULL)
-		if ((rsdp_addr = fwts_acpi_find_rsdp_bios()) == NULL)
-			return FWTS_ERROR;
+	/* Check for RSDP as EFI, then BIOS, parsed klog, if not found, give up */
+	if (fw->rsdp)
+		rsdp_addr = (void*)fw->rsdp;
+	if (!rsdp_addr)
+		rsdp_addr = fwts_acpi_find_rsdp_efi();
+	if (!rsdp_addr)
+		rsdp_addr = fwts_acpi_find_rsdp_bios();
+	if (!rsdp_addr)
+		rsdp_addr = fwts_acpi_find_rsdp_klog();
+	if (!rsdp_addr)
+		return FWTS_ERROR;
 
 	/* Load and save cached RSDP */
 	if ((rsdp = fwts_acpi_get_rsdp(rsdp_addr, &rsdp_len)) == NULL)
 		return FWTS_ERROR;
-	fwts_acpi_add_table("RSDP", rsdp, (uint64_t)(off_t)rsdp_addr, rsdp_len, FWTS_ACPI_TABLE_FROM_FIRMWARE);
 
-	/* Load any tables from RSDT if it's valid */
-	if (rsdp->rsdt_address) {
-		if ((rsdt = fwts_acpi_load_table((off_t)rsdp->rsdt_address)) != NULL) {
-			fwts_acpi_add_table("RSDT", rsdt, (uint64_t)rsdp->rsdt_address,
-				rsdt->header.length, FWTS_ACPI_TABLE_FROM_FIRMWARE);
-			num_entries = (rsdt->header.length - sizeof(fwts_acpi_table_header)) / 4;
-			for (i=0; i<num_entries; i++) {
-				if (rsdt->entries[i]) {
-					if ((header = fwts_acpi_load_table((off_t)rsdt->entries[i])) != NULL) {
-						if (strncmp("FACP", header->signature, 4) == 0)
-							fwts_acpi_handle_fadt((fwts_acpi_table_fadt*)header,
-								FWTS_ACPI_TABLE_FROM_FIRMWARE);
-						fwts_acpi_add_table(header->signature, header, (uint64_t)rsdt->entries[i],
-							header->length, FWTS_ACPI_TABLE_FROM_FIRMWARE);
-					}
-				}
-			}
-		}
-	}
+	fwts_acpi_add_table("RSDP", rsdp, (uint64_t)(off_t)rsdp_addr, rsdp_len, FWTS_ACPI_TABLE_FROM_FIRMWARE);
 
 	/* Load any tables from XSDT if it's valid */
 	if (rsdp->xsdt_address) {
@@ -290,8 +399,11 @@ static int fwts_acpi_load_tables_from_firmware(void)
 				if (xsdt->entries[i]) {
 					if ((header = fwts_acpi_load_table((off_t)xsdt->entries[i])) != NULL) {
 						if (strncmp("FACP", header->signature, 4) == 0)
-							fwts_acpi_handle_fadt((fwts_acpi_table_fadt*)header,
-								FWTS_ACPI_TABLE_FROM_FIRMWARE);
+							if (fwts_acpi_handle_fadt(fw,
+							    (uint64_t)xsdt->entries[i],
+							    (fwts_acpi_table_fadt *)header,
+							    FWTS_ACPI_TABLE_FROM_FIRMWARE) != FWTS_OK)
+								goto fail;
 						fwts_acpi_add_table(header->signature, header, xsdt->entries[i],
 							header->length, FWTS_ACPI_TABLE_FROM_FIRMWARE);
 					}
@@ -300,7 +412,37 @@ static int fwts_acpi_load_tables_from_firmware(void)
 		}
 	}
 
+	/* Load any tables from RSDT if it's valid */
+	if (rsdp->rsdt_address) {
+		if ((rsdt = fwts_acpi_load_table((off_t)rsdp->rsdt_address)) != NULL) {
+			fwts_acpi_add_table("RSDT", rsdt, (uint64_t)rsdp->rsdt_address,
+				rsdt->header.length, FWTS_ACPI_TABLE_FROM_FIRMWARE);
+			num_entries = (rsdt->header.length - sizeof(fwts_acpi_table_header)) / 4;
+			for (i=0; i<num_entries; i++) {
+				if (rsdt->entries[i]) {
+					if ((header = fwts_acpi_load_table((off_t)rsdt->entries[i])) != NULL) {
+						if (strncmp("FACP", header->signature, 4) == 0)
+							if (fwts_acpi_handle_fadt(fw,
+							    (uint64_t)rsdt->entries[i],
+							    (fwts_acpi_table_fadt *)header,
+							    FWTS_ACPI_TABLE_FROM_FIRMWARE) != FWTS_OK)
+								goto fail;
+						fwts_acpi_add_table(header->signature, header, (uint64_t)rsdt->entries[i],
+							header->length, FWTS_ACPI_TABLE_FROM_FIRMWARE);
+					}
+				}
+			}
+		}
+	}
+
 	return FWTS_OK;
+fail:
+	/*
+	 * Free'ing the tables will cause acpica_init to fail
+	 * and so we abort any ACPI related tests
+	 */
+	fwts_acpi_free_tables();
+	return FWTS_ERROR;
 }
 
 /*
@@ -371,6 +513,7 @@ static uint8_t *fwts_acpi_load_table_from_acpidump(FILE *fp, char *name, uint64_
 
 	/* Pull in 16 bytes at a time */
 	while (fgets(buffer, sizeof(buffer), fp) ) {
+		uint8_t *new_tmp;
 		int n;
 		buffer[56] = '\0';	/* truncate */
 		if ((n = sscanf(buffer,"  %x: %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx",
@@ -382,8 +525,14 @@ static uint8_t *fwts_acpi_load_table_from_acpidump(FILE *fp, char *name, uint64_
 			break;
 
 		len += (n - 1);
-		if ((tmp = realloc(tmp, len)) == NULL)
+		if (len == 0)
+			break;	/* No data, must be corrupt input */
+		if ((new_tmp = realloc(tmp, len)) == NULL) {
+			free(tmp);
 			return NULL;
+		} else
+			tmp = new_tmp;
+
 		memcpy(tmp + offset, data, n-1);
 	}
 
@@ -578,44 +727,63 @@ static bool fwts_acpi_table_fixable(fwts_acpi_table_info *table)
  */
 static int fwts_acpi_load_tables_fixup(fwts_framework *fw)
 {
-	int i;
-	int j;
-	int count;
+	int i, j, count;
 	char *oem_tbl_id = "FWTS    ";
 	fwts_acpi_table_info *table;
 	fwts_acpi_table_rsdp *rsdp = NULL;
 	fwts_acpi_table_rsdt *rsdt = NULL;
 	fwts_acpi_table_xsdt *xsdt = NULL;
 	fwts_acpi_table_fadt *fadt = NULL;
+	fwts_acpi_table_facs *facs = NULL;
+	uint64_t rsdt_fake_addr = 0, xsdt_fake_addr = 0;
+	bool redo_rsdp_checksum = false;
 
 	/* Fetch the OEM Table ID */
-	if (fwts_acpi_find_table(fw, "FACP", 0, &table) != FWTS_OK)
+	if (fwts_acpi_find_table(fw, "FACP", 0, &table) != FWTS_OK) {
+		fwts_log_error(fw, "ACPI table find failure.");
 		return FWTS_ERROR;
+	}
 	if (table) {
 		fadt = (fwts_acpi_table_fadt *)table->data;
 		oem_tbl_id = fadt->header.oem_tbl_id;
+	} else {
+		fwts_log_error(fw, "Cannot find FACP.");
+		return FWTS_ERROR;
 	}
 
-	/* Get RSDP */
-	if (fwts_acpi_find_table(fw, "RSDP", 0, &table) != FWTS_OK)
+	/* Get FACS */
+	if (fwts_acpi_find_table(fw, "FACS", 0, &table) != FWTS_OK) {
+		fwts_log_error(fw, "ACPI table find failure.");
 		return FWTS_ERROR;
+	}
 	if (table)
-		rsdp = (fwts_acpi_table_rsdp *)table->data;
+		facs = (fwts_acpi_table_facs *)table->data;
+	else {
+		size_t size = 64;
+		uint64_t facs_fake_addr;
 
-	/* Get RSDT */
-	if (fwts_acpi_find_table(fw, "RSDT", 0, &table) != FWTS_OK)
-		return FWTS_ERROR;
-	if (table)
-		rsdt = (fwts_acpi_table_rsdt *)table->data;
+		/* This is most unexpected, so warn about it */
+		fwts_log_warning(fw, "No FACS found, fwts has faked one instead.");
+		if ((facs = fwts_low_calloc(1, size)) == NULL) {
+			fwts_log_error(fw, "Cannot allocate fake FACS.");
+			return FWTS_ERROR;
+		}
+		strncpy(facs->signature, "FACS", 4);
+		facs->length = size;
+		facs->hardware_signature = 0xf000a200;	/* Some signature */
+		facs->flags = 0;
+		facs->version = 2;
+		facs_fake_addr = fwts_fake_physical_addr(size);
+		fadt->firmware_control = (uint32_t)facs_fake_addr;
+		if (fadt->header.length >= 140)
+			fadt->x_firmware_ctrl = (uint64_t)facs_fake_addr;
 
-	/* Get XSDT */
-	if (fwts_acpi_find_table(fw, "XSDT", 0, &table) != FWTS_OK)
-		return FWTS_ERROR;
-	if (table)
-		xsdt = (fwts_acpi_table_xsdt *)table->data;
+		fwts_acpi_add_table("FACS", facs, (uint64_t)facs_fake_addr,
+			size, FWTS_ACPI_TABLE_FROM_FIXUP);
+	}
 
 	/* Figure out how many tables we need to put into RSDT and XSDT */
-	for (count=0,i=0;;i++) {
+	for (count = 0, i = 0; ; i++) {
 		if (fwts_acpi_get_table(fw, i, &table) != FWTS_OK)
 			break;
 		if (table == NULL)	/* No more tables */
@@ -624,11 +792,22 @@ static int fwts_acpi_load_tables_fixup(fwts_framework *fw)
 			count++;
 	}
 
-	/* No RSDT? go and fake one */
-	if (rsdt == NULL) {
+	/* Get RSDT */
+	if (fwts_acpi_find_table(fw, "RSDT", 0, &table) != FWTS_OK) {
+		fwts_log_error(fw, "ACPI table find failure.");
+		return FWTS_ERROR;
+	}
+	if (table) {
+		rsdt = (fwts_acpi_table_rsdt *)table->data;
+		rsdt_fake_addr = table->addr;
+	} else {
+		/* No RSDT? go and fake one */
 		size_t size = sizeof(fwts_acpi_table_rsdt) + (count * sizeof(uint32_t));
-		if ((rsdt = fwts_low_calloc(1, size)) == NULL)
+
+		if ((rsdt = fwts_low_calloc(1, size)) == NULL) {
+			fwts_log_error(fw, "Cannot allocate fake RSDT.");
 			return FWTS_ERROR;
+		}
 
 		for (i=0,j=0; j<count ;i++)
 			if (fwts_acpi_get_table(fw, i, &table) == FWTS_OK)
@@ -645,15 +824,26 @@ static int fwts_acpi_load_tables_fixup(fwts_framework *fw)
 		rsdt->header.creator_revision = 1;
 		rsdt->header.checksum = 256 - fwts_checksum((uint8_t*)rsdt, size);
 
-		fwts_acpi_add_table("RSDT", rsdt, (uint64_t)fwts_fake_physical_addr(size),
-			size, FWTS_ACPI_TABLE_FROM_FIXUP);
+		rsdt_fake_addr = fwts_fake_physical_addr(size);
+		fwts_acpi_add_table("RSDT", rsdt, rsdt_fake_addr, size, FWTS_ACPI_TABLE_FROM_FIXUP);
 	}
 
-	/* No XSDT? go and fake one */
-	if (xsdt == NULL) {
+	/* Get XSDT */
+	if (fwts_acpi_find_table(fw, "XSDT", 0, &table) != FWTS_OK) {
+		fwts_log_error(fw, "ACPI table find failure.");
+		return FWTS_ERROR;
+	}
+	if (table) {
+		xsdt = (fwts_acpi_table_xsdt *)table->data;
+		xsdt_fake_addr = table->addr;
+	} else {
+		/* No XSDT? go and fake one */
 		size_t size = sizeof(fwts_acpi_table_rsdt) + (count * sizeof(uint64_t));
-		if ((xsdt = fwts_low_calloc(1, size)) == NULL)
+
+		if ((xsdt = fwts_low_calloc(1, size)) == NULL) {
+			fwts_log_error(fw, "Cannot allocate fake XSDT.");
 			return FWTS_ERROR;
+		}
 
 		for (i=0,j=0; j<count ;i++)
 			if (fwts_acpi_get_table(fw, i, &table) == FWTS_OK)
@@ -670,32 +860,57 @@ static int fwts_acpi_load_tables_fixup(fwts_framework *fw)
 		xsdt->header.creator_revision = 1;
 		xsdt->header.checksum = 256 - fwts_checksum((uint8_t*)xsdt, size);
 
-		fwts_acpi_add_table("XSDT", xsdt, (uint64_t)fwts_fake_physical_addr(size),
-			size, FWTS_ACPI_TABLE_FROM_FIXUP);
+		xsdt_fake_addr = fwts_fake_physical_addr(size);
+		fwts_acpi_add_table("XSDT", xsdt, xsdt_fake_addr, size, FWTS_ACPI_TABLE_FROM_FIXUP);
 	}
 
-	/* No RSDP? go and fake one */
-	if (rsdp == NULL) {
+	/* Get RSDP */
+	if (fwts_acpi_find_table(fw, "RSDP", 0, &table) != FWTS_OK)
+		return FWTS_ERROR;
+	if (table)
+		rsdp = (fwts_acpi_table_rsdp *)table->data;
+	else {
+		/* No RSDP? go and fake one */
 		size_t size = sizeof(fwts_acpi_table_rsdp);
-		if ((rsdp = fwts_low_calloc(1, size)) == NULL)
+
+		if ((rsdp = fwts_low_calloc(1, size)) == NULL) {
+			fwts_log_error(fw, "Cannot allocate fake RSDP.");
 			return FWTS_ERROR;
+		}
 
 		strncpy(rsdp->signature, "RSD PTR ", 8);
 		strncpy(rsdp->oem_id, "FWTS  ", 6);
 		rsdp->revision = 2;
-		rsdp->rsdt_address = (unsigned long)rsdt;
 		rsdp->length = sizeof(fwts_acpi_table_rsdp);
-		rsdp->xsdt_address = (uint64_t)(unsigned long)xsdt;
 		rsdp->reserved[0] = 0;
 		rsdp->reserved[1] = 0;
 		rsdp->reserved[2] = 0;
 
 		rsdp->checksum = 256 - fwts_checksum((uint8_t*)rsdp, 20);
-		rsdp->extended_checksum = 256 - fwts_checksum((uint8_t*)rsdp, sizeof(fwts_acpi_table_rsdp));
+		rsdp->extended_checksum = 256 - fwts_checksum((uint8_t*)rsdp, rsdp->length);
 
 		fwts_acpi_add_table("RSDP", rsdp, (uint64_t)fwts_fake_physical_addr(size),
 			sizeof(fwts_acpi_table_rsdp), FWTS_ACPI_TABLE_FROM_FIXUP);
 	}
+
+	/* Now we have all the tables, final fix up is required */
+	if (rsdp->rsdt_address != rsdt_fake_addr) {
+		rsdp->rsdt_address = rsdt_fake_addr;
+		redo_rsdp_checksum = true;
+	}
+	if ((rsdp->revision > 0) && (rsdp->length >= 36) &&
+	    (rsdp->xsdt_address != xsdt_fake_addr)) {
+		rsdp->xsdt_address = xsdt_fake_addr;
+		redo_rsdp_checksum = true;
+	}
+	/* And update checksum if we've updated the rsdp */
+	if (redo_rsdp_checksum) {
+		rsdp->checksum = 0;	/* Clear old checksum */
+		rsdp->checksum = 256 - fwts_checksum((uint8_t*)rsdp, 20);
+		rsdp->extended_checksum = 0;	/* Clear old checksum */
+		rsdp->extended_checksum = 256 - fwts_checksum((uint8_t*)rsdp, rsdp->length);
+	}
+
 	return FWTS_OK;
 }
 
@@ -712,16 +927,18 @@ int fwts_acpi_load_tables(fwts_framework *fw)
 	else if (fw->acpi_table_acpidump_file != NULL)
 		ret = fwts_acpi_load_tables_from_acpidump(fw);
 	else if (fwts_check_root_euid(fw, true) == FWTS_OK)
-		ret = fwts_acpi_load_tables_from_firmware();
+		ret = fwts_acpi_load_tables_from_firmware(fw);
 	else
 		ret = FWTS_ERROR_NO_PRIV;
 
 	if (ret == FWTS_OK) {
-		acpi_tables_loaded = 1;
+		acpi_tables_loaded = ACPI_TABLES_LOADED_OK;
 
 		/* Loading from file may require table address fixups */
 		if ((fw->acpi_table_path != NULL) || (fw->acpi_table_acpidump_file != NULL))
 			fwts_acpi_load_tables_fixup(fw);
+	} else {
+		acpi_tables_loaded = ACPI_TABLES_LOADED_FAILED;
 	}
 
 	return ret;
@@ -742,7 +959,7 @@ int fwts_acpi_find_table(fwts_framework *fw, const char *name, const int which,
 
 	*info = NULL;
 
-	if (!acpi_tables_loaded)
+	if (acpi_tables_loaded == ACPI_TABLES_NOT_LOADED)
 		if ((ret = fwts_acpi_load_tables(fw)) != FWTS_OK)
 			return ret;
 
@@ -772,7 +989,7 @@ int fwts_acpi_find_table_by_addr(fwts_framework *fw, const uint64_t addr, fwts_a
 
 	*info = NULL;
 
-	if (!acpi_tables_loaded)
+	if (acpi_tables_loaded == ACPI_TABLES_NOT_LOADED)
 		if ((ret = fwts_acpi_load_tables(fw)) != FWTS_OK)
 			return ret;
 
@@ -803,7 +1020,7 @@ int fwts_acpi_get_table(fwts_framework *fw, const int index, fwts_acpi_table_inf
 	if ((index < 0) || (index >= ACPI_MAX_TABLES))
 		return FWTS_ERROR;
 
-	if (!acpi_tables_loaded)
+	if (acpi_tables_loaded == ACPI_TABLES_NOT_LOADED)
 		if ((ret = fwts_acpi_load_tables(fw)) != FWTS_OK)
 			return ret;
 
