@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2013 Canonical
+ * Copyright (C) 2011-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -129,15 +129,20 @@ int fwts_args_parse(fwts_framework *fw, const int argc, char * const argv[])
 
 			if (short_name && (len = strlen(short_name)) > 0) {
 				if (short_options) {
-					short_options = realloc(short_options,
+					char *new_short_options;
+
+					new_short_options = realloc(short_options,
 						short_options_len + len + 1);
-					if (short_options == NULL) {
+					if (new_short_options == NULL) {
+						free(short_options);
 						free(long_options);
 						fwts_log_error(fw,
 							"Out of memory "
 							"allocating options.");
 						return FWTS_ERROR;
-					}
+					} else
+						short_options = new_short_options;
+
 					strcat(short_options, short_name);
 					short_options_len += (len + 1);
 				} else {
@@ -259,7 +264,7 @@ void fwts_args_show_options(void)
 
 	fwts_list_init(&sorted_options);
 
-	width = fwts_tty_width(fileno(stderr), FWTS_MIN_TTY_WIDTH);
+	width = fwts_tty_width(fileno(stdin), FWTS_MIN_TTY_WIDTH);
 	if ((width - (FWTS_ARGS_WIDTH + 1)) < 0)
 		width = FWTS_MIN_TTY_WIDTH;
 
