@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -70,12 +70,10 @@ static fwts_option fwts_framework_options[] = {
 	{ "skip-test", 		"S:", 1, "Skip listed tests, e.g. --skip-test=s3,nx,method" },
 	{ "quiet", 		"q",  0, "Run quietly." },
 	{ "dumpfile", 		"",   1, "Load ACPI tables using file generated by acpidump, e.g. --dumpfile=acpidump.dat" },
-	{ "lp-tags", 		"",   0, "Output just LaunchPad bug tags." },
 	{ "show-tests-full", 	"",   0, "Show available tests including all minor tests." },
 	{ "utils", 		"u",  0, "Run Utility 'tests'." },
 	{ "json-data-path", 	"j:", 1, "Specify path to fwts json data files - default is /usr/share/fwts." },
-	{ "lp-tags-log", 	"",   0, "Output LaunchPad bug tags in results log." },
-	{ "disassemble-aml", 	"",   0, "Disassemble AML from DSDT and SSDT tables." },
+	{ "disassemble-aml", 	"",   2, "Disassemble AML from DSDT and SSDT tables." },
 	{ "log-type",		"",   1, "Specify log type (plaintext, json, html or xml)." },
 	{ "unsafe",		"U",  0, "Unsafe tests (tests that can potentially cause kernel oopses)." },
 	{ "filter-error-discard", "", 1, "Discard errors that match any of the specified labels." },
@@ -83,14 +81,15 @@ static fwts_option fwts_framework_options[] = {
 	{ "acpica-debug",	"",   0, "Enable ACPICA debug/warning messages." },
 	{ "acpica",		"",   1, "Enable ACPICA run time options." },
 	{ "uefi",		"",   0, "Run UEFI tests." },
+	{ "rsdp",		"R:", 1, "Specify the physical address of the ACPI RSDP." },
 	{ NULL, NULL, 0, NULL }
 };
 
 static fwts_list fwts_framework_test_list = FWTS_LIST_INIT;
 
 static const char *fwts_copyright[] = {
-	"Some of this work - Copyright (c) 1999 - 2010, Intel Corp. All rights reserved.",
-	"Some of this work - Copyright (c) 2010 - 2013, Canonical.",
+	"Some of this work - Copyright (c) 1999 - 2014, Intel Corp. All rights reserved.",
+	"Some of this work - Copyright (c) 2010 - 2014, Canonical.",
 	NULL
 };
 
@@ -319,7 +318,9 @@ static void fwts_framework_format_results(char *buffer, int buflen, fwts_results
 		buflen -= n;
 	}
 	if ((include_zero_results || (results->warning > 0)) && (buflen > 0)) {
-		n = snprintf(buffer, buflen, "%s%u warnings", n > 0 ? ", " : "", results->warning);
+		n = snprintf(buffer, buflen, "%s%u warning%s",
+			n > 0 ? ", " : "", results->warning,
+			results->warning > 1 ? "s" : "");
 		buffer += n;
 		buflen -= n;
 	}
@@ -427,11 +428,8 @@ static int fwts_framework_test_summary(fwts_framework *fw)
 	fwts_framework_underline(fw,'=');
 
 	if (fw->flags & FWTS_FLAG_STDOUT_SUMMARY) {
-		if (results->aborted > 0)
-			printf("%s\n", fwts_log_field_to_str_upper(LOG_ABORTED));
-		else if (results->skipped > 0)
-			printf("%s\n", fwts_log_field_to_str_upper(LOG_SKIPPED));
-		else if (results->failed > 0) {
+		/* Report in order of failure precedence */
+		if (results->failed > 0) {
 			/* We intentionally report the highest logged error level */
 			if (fw->failed_level & LOG_LEVEL_CRITICAL)
 				printf("%s_CRITICAL\n", fwts_log_field_to_str_upper(LOG_FAILED));
@@ -442,15 +440,17 @@ static int fwts_framework_test_summary(fwts_framework *fw)
 			else if (fw->failed_level & LOG_LEVEL_LOW)
 				printf("%s_LOW\n", fwts_log_field_to_str_upper(LOG_FAILED));
 			else printf("%s\n", fwts_log_field_to_str_upper(LOG_FAILED));
-		}
+		} else if (results->skipped > 0)
+			printf("%s\n", fwts_log_field_to_str_upper(LOG_SKIPPED));
 		else if (results->warning > 0)
 			printf("%s\n", fwts_log_field_to_str_upper(LOG_WARNING));
+		else if (results->aborted > 0)
+			printf("%s\n", fwts_log_field_to_str_upper(LOG_ABORTED));
 		else
 			printf("%s\n", fwts_log_field_to_str_upper(LOG_PASSED));
 	}
 
-	if (!(fw->flags & FWTS_FLAG_LP_TAGS))
-		fwts_log_newline(fw->results);
+	fwts_log_newline(fw->results);
 
 	return FWTS_OK;
 }
@@ -472,7 +472,6 @@ static int fwts_framework_run_test(fwts_framework *fw, fwts_framework_test *test
 
 	fw->current_major_test = test;
 	fw->current_minor_test_name = "";
-	fwts_list_init(&fw->test_taglist);
 
 	test->was_run = true;
 	fw->total_run++;
@@ -594,15 +593,7 @@ static int fwts_framework_run_test(fwts_framework *fw, fwts_framework_test *test
 	if (test->ops->deinit)
 		test->ops->deinit(fw);
 
-	if (fw->flags & FWTS_FLAG_LP_TAGS_LOG) {
-		fwts_log_section_begin(fw->results, "tags");
-		fwts_tag_report(fw, LOG_TAG, &fw->test_taglist);
-		fwts_log_section_end(fw->results);
-	}
-
 done:
-	fwts_list_free_items(&fw->test_taglist, free);
-
 	if (!(test->flags & FWTS_FLAG_UTILS)) {
 		fwts_log_section_begin(fw->results, "results");
 		fwts_framework_test_summary(fw);
@@ -869,8 +860,7 @@ static void fwts_framework_heading_info(
 
 		fwts_log_info(fw, "Running tests: %s.",
 			fwts_list_len(tests_to_run) == 0 ? "None" : tests);
-		if (!(fw->flags & FWTS_FLAG_LP_TAGS))
-			fwts_log_newline(fw->results);
+		fwts_log_newline(fw->results);
 		free(tests);
 	}
 }
@@ -1091,51 +1081,46 @@ int fwts_framework_options_handler(fwts_framework *fw, int argc, char * const ar
 		case 25: /* --dumpfile */
 			fwts_framework_strdup(&fw->acpi_table_acpidump_file, optarg);
 			break;
-		case 26: /* --lp-tags */
-			fw->flags |= FWTS_FLAG_LP_TAGS;
-			fwts_log_filter_unset_field(~0);
-			fwts_log_filter_set_field(LOG_TAG);
-			break;
-		case 27: /* --show-tests-full */
+		case 26: /* --show-tests-full */
 			fw->flags |= FWTS_FLAG_SHOW_TESTS_FULL;
 			break;
-		case 28: /* --utils */
+		case 27: /* --utils */
 			fw->flags |= FWTS_FLAG_UTILS;
 			break;
-		case 29: /* --json-data-path */
+		case 28: /* --json-data-path */
 			fwts_framework_strdup(&fw->json_data_path, optarg);
 			break;
-		case 30: /* --lp-tags-log */
-			fw->flags |= FWTS_FLAG_LP_TAGS_LOG;
-			break;
-		case 31: /* --disassemble-aml */
-			fwts_iasl_disassemble_all_to_file(fw);
+		case 29: /* --disassemble-aml */
+			fwts_iasl_disassemble_all_to_file(fw, optarg);
 			return FWTS_COMPLETE;
-		case 32: /* --log-type */
+		case 30: /* --log-type */
 			if (fwts_framework_log_type_parse(fw, optarg) != FWTS_OK)
 				return FWTS_ERROR;
 			break;
-		case 33: /* --unsafe */
+		case 31: /* --unsafe */
 			fw->flags |= FWTS_FLAG_UNSAFE;
 			break;
-		case 34: /* --filter-error-discard */
+		case 32: /* --filter-error-discard */
 			if (fwts_framework_filter_error_parse(optarg, &fw->errors_filter_discard) != FWTS_OK)
 				return FWTS_ERROR;
 			break;
-		case 35: /* --filter-error-keep */
+		case 33: /* --filter-error-keep */
 			if (fwts_framework_filter_error_parse(optarg, &fw->errors_filter_keep) != FWTS_OK)
 				return FWTS_ERROR;
 			break;
-		case 36: /* --acpica-debug */
+		case 34: /* --acpica-debug */
 			fw->flags |= FWTS_FLAG_ACPICA_DEBUG;
 			break;
-		case 37: /* --acpica */
+		case 35: /* --acpica */
 			if (fwts_framework_acpica_parse(fw, optarg) != FWTS_OK)
 				return FWTS_ERROR;
 			break;
-		case 38: /* --uefi */
+		case 36: /* --uefi */
 			fw->flags |= FWTS_FLAG_TEST_UEFI;
 			break;
+		case 37: /* --rsdp */
+			fw->rsdp = (void *)strtoul(optarg, NULL, 0);
+			break;
 		}
 		break;
 	case 'a': /* --all */
@@ -1170,7 +1155,6 @@ int fwts_framework_options_handler(fwts_framework *fw, int argc, char * const ar
 		fwts_framework_strdup(&fw->klog, optarg);
 		break;
 	case 'l': /* --lp-flags */
-		fw->flags |= FWTS_FLAG_LP_TAGS;
 		break;
 	case 'p': /* --show-progress */
 		fw->flags = (fw->flags &
@@ -1212,6 +1196,9 @@ int fwts_framework_options_handler(fwts_framework *fw, int argc, char * const ar
 	case 'w': /* --log-width=N */
 		fwts_log_set_line_width(atoi(optarg));
 		break;
+	case 'R': /* --rsdp=addr */
+		fw->rsdp = (void *)strtoul(optarg, NULL, 0);
+		break;
 	}
 	return FWTS_OK;
 }
@@ -1243,7 +1230,6 @@ int fwts_framework_args(const int argc, char **argv)
 		    FWTS_FLAG_SHOW_PROGRESS;
 	fw->log_type = LOG_TYPE_PLAINTEXT;
 
-	fwts_list_init(&fw->total_taglist);
 	fwts_list_init(&fw->errors_filter_keep);
 	fwts_list_init(&fw->errors_filter_discard);
 
@@ -1374,17 +1360,12 @@ int fwts_framework_args(const int argc, char **argv)
 		fwts_log_section_begin(fw->results, "summary");
 		fwts_log_set_owner(fw->results, "summary");
 		fwts_log_nl(fw);
-		if (fw->flags & FWTS_FLAG_LP_TAGS_LOG)
-			fwts_tag_report(fw, LOG_SUMMARY, &fw->total_taglist);
 		fwts_framework_total_summary(fw);
 		fwts_log_nl(fw);
 		fwts_summary_report(fw, &fwts_framework_test_list);
 		fwts_log_section_end(fw->results);
 	}
 
-	if (fw->flags & FWTS_FLAG_LP_TAGS)
-		fwts_tag_report(fw, LOG_TAG | LOG_NO_FIELDS, &fw->total_taglist);
-
 tidy:
 	fwts_list_free_items(&tests_to_skip, NULL);
 	fwts_list_free_items(&tests_to_run, NULL);
@@ -1401,7 +1382,6 @@ tidy_close:
 
 	fwts_list_free_items(&fw->errors_filter_discard, NULL);
 	fwts_list_free_items(&fw->errors_filter_keep, NULL);
-	fwts_list_free_items(&fw->total_taglist, free);
 	fwts_list_free_items(&fwts_framework_test_list, free);
 
 	/* Failed tests flagged an error */
