@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -137,26 +137,34 @@ int fwts_iasl_disassemble(fwts_framework *fw,
  *  fwts_iasl_disassemble_all_to_file()
  * 	Disassemble DSDT and SSDT tables to separate files.
  */
-int fwts_iasl_disassemble_all_to_file(fwts_framework *fw)
+int fwts_iasl_disassemble_all_to_file(fwts_framework *fw,
+	const char *path)
 {
 	int i;
 	int ret;
+	char filename[PATH_MAX];
+	char pathname[PATH_MAX];
 
-	ret = fwts_iasl_disassemble_to_file(fw, "DSDT", 0, "DSDT.dsl");
+	if (path == NULL)
+		strncpy(pathname, "", sizeof(pathname));
+	else
+		snprintf(pathname, sizeof(pathname), "%s/", path);
+
+	snprintf(filename, sizeof(filename), "%sDSDT.dsl", pathname);
+
+	ret = fwts_iasl_disassemble_to_file(fw, "DSDT", 0, filename);
 	if (ret == FWTS_ERROR_NO_PRIV) {
 		fprintf(stderr, "Need to have root privilege to read ACPI tables from memory! Re-run using sudo.\n");
 		return FWTS_ERROR;
 	}
 	if (ret == FWTS_OK)
-		printf("Disassembled DSDT to DSDT.dsl\n");
+		printf("Disassembled DSDT to %s\n", filename);
 
 	for (i=0; ;i++) {
-		char filename[PATH_MAX];
-
-		snprintf(filename, sizeof(filename), "SSDT%d.dsl", i);
+		snprintf(filename, sizeof(filename), "%sSSDT%d.dsl", pathname, i);
 		if (fwts_iasl_disassemble_to_file(fw, "SSDT", i, filename) != FWTS_OK)
 			break;
-		printf("Disassembled SSDT %d to SSDT%d.dsl\n", i, i);
+		printf("Disassembled SSDT %d to %s\n", i, filename);
 	}
 
 	return FWTS_OK;
@@ -172,14 +180,17 @@ int fwts_iasl_reassemble(fwts_framework *fw,
 	const uint8_t *data,
 	const int len,
 	fwts_list **iasl_disassembly,
-	fwts_list **iasl_errors)
+	fwts_list **iasl_stdout,
+	fwts_list **iasl_stderr)
 {
 	char tmpfile[PATH_MAX];
 	char amlfile[PATH_MAX];
-	char *output_text = NULL;
+	char *stdout_output = NULL, *stderr_output = NULL;
 	int pid = getpid();
 
-	if ((iasl_disassembly  == NULL) || (iasl_errors == NULL))
+	if ((iasl_disassembly  == NULL) ||
+	    (iasl_stdout == NULL) ||
+	    (iasl_stderr == NULL))
 		return FWTS_ERROR;
 
 	fwts_acpcia_set_fwts_framework(fw);
@@ -205,10 +216,10 @@ int fwts_iasl_reassemble(fwts_framework *fw,
 
 	/* Now we have a disassembled source in tmpfile, so let's assemble it */
 
-	if (fwts_iasl_assemble_aml(tmpfile, &output_text) < 0) {
+	if (fwts_iasl_assemble_aml(tmpfile, &stdout_output, &stderr_output) < 0) {
 		(void)unlink(amlfile);
 		(void)unlink(tmpfile);
-		free(output_text);
+		free(stdout_output);
 		return FWTS_ERROR;
 	}
 
@@ -220,8 +231,9 @@ int fwts_iasl_reassemble(fwts_framework *fw,
 	snprintf(tmpfile, sizeof(tmpfile), "/tmp/fwts_iasl_%d.aml", pid);
 	(void)unlink(tmpfile);
 
-	*iasl_errors = fwts_list_from_text(output_text);
-	free(output_text);
+	*iasl_stdout = fwts_list_from_text(stdout_output);
+	*iasl_stderr = fwts_list_from_text(stderr_output);
+	free(stdout_output);
 
 	return FWTS_OK;
 }
