@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -75,8 +75,6 @@ char *fwts_log_field_to_str(const fwts_log_field field)
 		return "ADV";
 	case LOG_HEADING:
 		return "HED";
-	case LOG_TAG:
-		return "TAG";
 	case LOG_PASSED:
 		return "PAS";
 	case LOG_FAILED:
@@ -119,8 +117,6 @@ char *fwts_log_field_to_str_full(const fwts_log_field field)
 		return "Advice";
 	case LOG_HEADING:
 		return "Heading";
-	case LOG_TAG:
-		return "Tag";
 	case LOG_PASSED:
 		return "Passed";
 	case LOG_FAILED:
@@ -237,9 +233,14 @@ fwts_log_field fwts_log_str_to_field(const char *text)
 		{ "INF", LOG_INFO },
 		{ "SUM", LOG_SUMMARY },
 		{ "SEP", LOG_SEPARATOR },
+		{ "NLN", LOG_NEWLINE },
 		{ "ADV", LOG_ADVICE },
 		{ "HED", LOG_HEADING },
-		{ "TAG", LOG_TAG },
+		{ "PAS", LOG_PASSED },
+		{ "FAL", LOG_FAILED },
+		{ "SKP", LOG_SKIPPED },
+		{ "ABT", LOG_ABORTED },
+		{ "INO", LOG_INFOONLY },
 		{ "ALL", ~0 },
 		{ NULL, 0 }
 	};
@@ -583,11 +584,16 @@ char *fwts_log_get_filenames(const char *filename, const fwts_log_type type)
 			}
 
 			if (filenames) {
+				char *new_filenames;
+
 				len += strlen(tmp) + 2;
-				if ((filenames = realloc(filenames, len)) == NULL) {
+				if ((new_filenames = realloc(filenames, len)) == NULL) {
+					free(filenames);
 					free(tmp);
 					return NULL;
-				}
+				} else
+					filenames = new_filenames;
+
 				strcat(filenames, " ");
 				strcat(filenames, tmp);
 			} else {
