@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Canonical
+ * Copyright (C) 2010-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -97,9 +97,15 @@ char *fwts_pipe_read(const int fd, ssize_t *length)
 			}
 		}
 		else {
-			ptr = realloc(ptr, size + n + 1);
-			if (ptr == NULL)
+			char *new_ptr;
+
+			new_ptr = realloc(ptr, size + n + 1);
+			if (new_ptr == NULL) {
+				free(ptr);
 				return NULL;
+			} else
+				ptr = new_ptr;
+
 			memcpy(ptr + size, buffer, n);
 			size += n;
 			*(ptr+size) = 0;
@@ -158,3 +164,23 @@ int fwts_pipe_exec(const char *command, fwts_list **list, int *status)
 	}
 	return FWTS_OK;
 }
+
+/*
+ *  fwts_exec()
+ *	execute a command
+ *	Return FWTS_OK if the exec worked, FWTS_EXEC_ERROR if
+ *	it failed.  status contains the child exit status.
+ */
+int fwts_exec(const char *command, int *status)
+{
+	pid_t 	pid;
+	int	fd;
+
+	if ((fd = fwts_pipe_open(command, &pid)) < 0)
+		return FWTS_ERROR;
+
+	*status = fwts_pipe_close(fd, pid);
+	if (*status)
+		return FWTS_EXEC_ERROR;
+	return FWTS_OK;
+}
