@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2013 Canonical
+ * Copyright (C) 2011-2014 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,27 +18,19 @@
  */
 #include <stdlib.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
+#include <dirent.h>
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
 #include <limits.h>
+#include <inttypes.h>
 #include "fwts.h"
 
-/* PCI Confiiguration Space for Type 0 & 1 */
-#define FWTS_PCI_HEADER_TYPE		0x0E
-#define FWTS_PCI_CAPABILITIES_POINTER	0x34
-
-/* PCI Confiiguration Space for Type 1 */
-#define FWTS_PCI_SECONDARD_BUS_NUMBER	0x19
-
-/* PCI Capability IDs  */
-#define FWTS_PCI_LAST_ID		0x00
-#define FWTS_PCI_EXPRESS_CAP_ID		0x10
-
 /* PCI Express Capability Structure Fields */
 #define FWTS_PCIE_ASPM_SUPPORT_L0_FIELD	0x0400
 #define FWTS_PCIE_ASPM_SUPPORT_L1_FIELD	0x0800
@@ -48,47 +40,15 @@
 #define FWTS_PCIE_ASPM_CONTROL_L1_FIELD	0x0002
 #define FWTS_PCIE_ASPM_CONTROL_FIELD	(FWTS_PCIE_ASPM_CONTROL_L0_FIELD | FWTS_PCIE_ASPM_CONTROL_L1_FIELD)
 
-
 struct pci_device {
 	uint8_t segment;
 	uint8_t bus;
 	uint8_t dev;
 	uint8_t func;
 	uint8_t config[256];
-	struct pci_device *next;
 };
 
-/* PCI Express Capability Structure is defined in Section 7.8
- * of PCI Express®i Base Specification Revision 2.0
- */
-typedef struct {
-	uint8_t pcie_cap_id;
-	uint8_t next_cap_point;
-	uint16_t pcie_cap_reg;
-	uint32_t device_cap;
-	uint16_t device_contrl;
-	uint16_t device_status;
-	uint32_t link_cap;
-	uint16_t link_contrl;
-	uint16_t link_status;
-	uint32_t slot_cap;
-	uint16_t slot_contrl;
-	uint16_t slot_status;
-	uint16_t root_contrl;
-	uint16_t root_cap;
-	uint32_t root_status;
-	uint32_t device_cap2;
-	uint16_t device_contrl2;
-	uint16_t device_status2;
-	uint32_t link_cap2;
-	uint16_t link_contrl2;
-	uint16_t link_status2;
-	uint32_t slot_cap2;
-	uint16_t slot_contrl2;
-	uint16_t slot_status2;
-} __attribute__ ((packed)) pcie_capability;
-
-static int facp_get_aspm_control(fwts_framework *fw, int *aspm)
+static int facp_get_aspm_control(fwts_framework *fw)
 {
 	fwts_acpi_table_info *table;
 	fwts_acpi_table_fadt *fadt;
@@ -99,10 +59,8 @@ static int facp_get_aspm_control(fwts_framework *fw, int *aspm)
 	fadt = (fwts_acpi_table_fadt*)table->data;
 
 	if ((fadt->iapc_boot_arch & FWTS_FACP_IAPC_BOOT_ARCH_PCIE_ASPM_CONTROLS) == 0) {
-		*aspm = 1;
 		fwts_log_info(fw, "PCIe ASPM is controlled by Linux kernel.");
 	} else {
-		*aspm = 0;
 		fwts_log_info(fw, "PCIe ASPM is not controlled by Linux kernel.");
 		fwts_advice(fw,
 			"BIOS reports that Linux kernel should not modify ASPM "
@@ -118,28 +76,28 @@ static int pcie_compare_rp_dev_aspm_registers(fwts_framework *fw,
 	struct pci_device *rp,
 	struct pci_device *dev)
 {
-	pcie_capability *rp_cap, *device_cap;
+	fwts_pcie_capability *rp_cap, *device_cap;
 	uint8_t rp_aspm_cntrl, device_aspm_cntrl;
 	uint8_t next_cap;
 	int ret = FWTS_OK;
 	bool l0s_disabled = false, l1_disabled = false;
 
-	next_cap = rp->config[FWTS_PCI_CAPABILITIES_POINTER];
-	rp_cap = (pcie_capability *) &rp->config[next_cap];
+	next_cap = rp->config[FWTS_PCI_CONFIG_TYPE1_CAPABILITY_POINTER];
+	rp_cap = (fwts_pcie_capability *) &rp->config[next_cap];
 	while (rp_cap->pcie_cap_id != FWTS_PCI_EXPRESS_CAP_ID) {
-		if (rp_cap->next_cap_point == FWTS_PCI_LAST_ID)
+		if (rp_cap->next_cap_point == FWTS_PCI_CAPABILITIES_LAST_ID)
 			break;
 		next_cap = rp_cap->next_cap_point;
-		rp_cap = (pcie_capability *) &rp->config[next_cap];
+		rp_cap = (fwts_pcie_capability *) &rp->config[next_cap];
 	}
 
-	next_cap = dev->config[FWTS_PCI_CAPABILITIES_POINTER];
-	device_cap = (pcie_capability *)&dev->config[next_cap];
+	next_cap = dev->config[FWTS_PCI_CONFIG_TYPE1_CAPABILITY_POINTER];
+	device_cap = (fwts_pcie_capability *)&dev->config[next_cap];
 	while (device_cap->pcie_cap_id != FWTS_PCI_EXPRESS_CAP_ID) {
-		if (device_cap->next_cap_point == FWTS_PCI_LAST_ID)
+		if (device_cap->next_cap_point == FWTS_PCI_CAPABILITIES_LAST_ID)
 			break;
 		next_cap = device_cap->next_cap_point;
-		device_cap = (pcie_capability *)&dev->config[next_cap];
+		device_cap = (fwts_pcie_capability *)&dev->config[next_cap];
 	}
 
 
@@ -211,126 +169,84 @@ static int pcie_compare_rp_dev_aspm_registers(fwts_framework *fw,
 	return ret;
 }
 
-static void pci_device_list_free(struct pci_device *head)
-{
-	struct pci_device *next;
-
-	while (head) {
-		next = head->next;
-		free(head);
-		head = next;
-	}
-}
-
 static int pcie_check_aspm_registers(fwts_framework *fw)
 {
-	fwts_list *lspci_output;
-	fwts_list_link *item;
-	struct pci_device *head = NULL, *cur = NULL, *device = NULL;
-	char command[PATH_MAX];
-	int ret = FWTS_OK;
-	int status;
+	DIR *dirp;
+	struct dirent *entry;
+	fwts_list dev_list;
+	fwts_list_link *lcur, *ltarget;
 
-	snprintf(command, sizeof(command), "%s", fw->lspci);
+	fwts_list_init(&dev_list);
 
-	if (fwts_pipe_exec(command, &lspci_output, &status) != FWTS_OK) {
-		fwts_log_warning(fw, "Could not execute %s.", command);
-		return FWTS_ERROR;
-	}
-	if (lspci_output == NULL) {
-		fwts_log_warning(fw, "Output from lspci was empty.");
+	if ((dirp = opendir(FWTS_PCI_DEV_PATH)) == NULL) {
+		fwts_log_warning(fw, "Could not open %s.", FWTS_PCI_DEV_PATH);
 		return FWTS_ERROR;
 	}
+	while ((entry = readdir(dirp)) != NULL) {
+		uint8_t bus, dev, func;
 
-	/* Get the list of pci devices and their configuration */
-	fwts_list_foreach(item, lspci_output) {
-		char *line = fwts_text_list_text(item);
-		char *pEnd;
+		if (entry->d_name[0] == '.')
+			continue;
 
-		device = (struct pci_device *) calloc(1, sizeof(*device));
-		if (device == NULL) {
-			pci_device_list_free(head);
-			fwts_text_list_free(lspci_output);
-			return FWTS_ERROR;
-		}
+		if (sscanf(entry->d_name, "%*x:%" SCNx8 ":%" SCNx8 ".%" SCNx8, &bus, &dev, &func) == 3) {
+			int fd;
+			char path[PATH_MAX];
+			struct pci_device *device;
 
-		device->bus = strtol(line, &pEnd, 16);
-		device->dev = strtol(pEnd + 1, &pEnd, 16);
-		device->func = strtol(pEnd + 1, &pEnd, 16);
-
-		if (head == NULL)
-			head = device;
-		else
-			cur->next = device;
-
-		cur = device;
-	}
-	fwts_text_list_free(lspci_output);
-
-	for (cur = head; cur; cur = cur->next) {
-		int reg_loc = 0, reg_val = 0;
-		int i;
-		int status;
-
-		snprintf(command, sizeof(command), "%s -s %02X:%02X.%02X -xxx",
-			fw->lspci, cur->bus, cur->dev, cur->func);
-		if (fwts_pipe_exec(command, &lspci_output, &status) != FWTS_OK) {
-			fwts_log_warning(fw, "Could not execute %s", command);
-			pci_device_list_free(head);
-			return FWTS_ERROR;
-		}
-		if (lspci_output == NULL) {
-			pci_device_list_free(head);
-			return FWTS_ERROR;
-		}
-
-		fwts_list_foreach(item, lspci_output) {
-			char *line = fwts_text_list_text(item);
-			char *pEnd;
-
-			if (strlen(line) >= 3 && line[3] == ' ') {
-				reg_val = strtol(line, &pEnd, 16);
-				for (i = 0; reg_loc < 256 && i < 16; i++) {
-					reg_val = strtol(pEnd + 1, &pEnd, 16);
-					cur->config[reg_loc] = reg_val;
-					reg_loc++;
-				}
+			device = (struct pci_device *)calloc(1, sizeof(struct pci_device));
+			if (device == NULL) {
+				fwts_list_free_items(&dev_list, free);
+				closedir(dirp);
+				return FWTS_ERROR;
 			}
-		}
-		fwts_text_list_free(lspci_output);
-	}
+			device->bus = bus;
+			device->dev = dev;
+			device->func = func;
 
-	/* Check aspm registers from the list of pci devices */
-	for (cur = head; cur; cur = cur->next) {
-		struct pci_device *target;
-
-		/* Find PCI Bridge (PCIE Root Port) and the attached device  */
-		if (cur->config[FWTS_PCI_HEADER_TYPE] & 0x01) {
-			target = head;
-			while (target != NULL) {
-				if (target->bus == cur->config[FWTS_PCI_SECONDARD_BUS_NUMBER])
-					break;
-				target = target->next;
-			}
-			if (target == NULL) {
+			snprintf(path, sizeof(path), FWTS_PCI_DEV_PATH "/%s/config", entry->d_name);
+			if ((fd = open(path, O_RDONLY)) < 0) {
+				fwts_log_warning(fw, "Could not open config from PCI device %s\n", entry->d_name);
+				free(device);
 				continue;
 			}
 
-			pcie_compare_rp_dev_aspm_registers(fw, cur, target);
+			if (read(fd, device->config, sizeof(device->config)) < 0) {
+				fwts_log_warning(fw, "Could not read config from PCI device %s\n", entry->d_name);
+				free(device);
+				close(fd);
+				continue;
+			}
+			close(fd);
+			fwts_list_append(&dev_list, device);
 		}
 	}
+	closedir(dirp);
 
-	pci_device_list_free(head);
+	/* Check aspm registers from the list of pci devices */
+	for (lcur = dev_list.head; lcur; lcur = lcur->next) {
+		struct pci_device *target, *cur = (struct pci_device *)lcur->data;
 
-	return ret;
+		/* Find PCI Bridge (PCIE Root Port) and the attached device  */
+		if (cur->config[FWTS_PCI_CONFIG_HEADER_TYPE] & 0x01) {
+			for (ltarget = dev_list.head; ltarget; ltarget = ltarget->next) {
+				target = (struct pci_device *)ltarget->data;
+				if (target->bus == cur->config[FWTS_PCI_CONFIG_TYPE1_SECONDARY_BUS_NUMBER]) {
+					pcie_compare_rp_dev_aspm_registers(fw, cur, target);
+					break;
+				}
+			}
+		}
+	}
+	fwts_list_free_items(&dev_list, free);
+
+	return FWTS_OK;
 }
 
 static int aspm_check_configuration(fwts_framework *fw)
 {
 	int ret;
-	int aspm_facp;
 
-	ret = facp_get_aspm_control(fw, &aspm_facp);
+	ret = facp_get_aspm_control(fw);
 	if (ret == FWTS_ERROR)
 		fwts_skipped(fw, "No valid FACP information present: cannot test ASPM.");
 
@@ -355,7 +271,7 @@ static fwts_framework_minor_test aspm_tests[] = {
 };
 
 static fwts_framework_ops aspm_ops = {
-	.description = "PCIe ASPM check.",
+	.description = "PCIe ASPM test.",
 	.minor_tests = aspm_tests
 };
 
